@@ -1,10 +1,18 @@
 //-----------------------------------------------------------------------------
-// Jonathan Westhues, April 2006
-// iZsh <izsh at fail0verflow.com>, 2014
+// Copyright (C) Jonathan Westhues, April 2006
+// Copyright (C) Proxmark3 contributors. See AUTHORS.md for details.
 //
-// This code is licensed to you under the terms of the GNU GPL, version 2 or,
-// at your option, any later version. See the LICENSE.txt file for the text of
-// the license.
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// See LICENSE.txt for the text of the license.
 //-----------------------------------------------------------------------------
 // Routines to load the FPGA image, and then to configure the FPGA's major
 // mode once it is configured.
@@ -17,21 +25,28 @@
 #include "ticks.h"
 #include "dbprint.h"
 #include "util.h"
-#include "zlib.h"
 #include "fpga.h"
 #include "string.h"
 
+#include "lz4.h"       // uncompress
+
+typedef struct {
+    LZ4_streamDecode_t *lz4StreamDecode;
+    char *next_in;
+    int avail_in;
+} lz4_stream_t;
+
+typedef lz4_stream_t *lz4_streamp_t;
+
 // remember which version of the bitstream we have already downloaded to the FPGA
-static int downloaded_bitstream = 0;
+static int downloaded_bitstream = FPGA_BITSTREAM_UNKNOWN;
 
 // this is where the bitstreams are located in memory:
-extern uint8_t _binary_obj_fpga_all_bit_z_start, _binary_obj_fpga_all_bit_z_end;
+extern uint32_t _binary_obj_fpga_all_bit_z_start[], _binary_obj_fpga_all_bit_z_end[];
 
 static uint8_t *fpga_image_ptr = NULL;
 static uint32_t uncompressed_bytes_cnt;
 
-#define OUTPUT_BUFFER_LEN 80
-
 //-----------------------------------------------------------------------------
 // Set up the Serial Peripheral Interface as master
 // Used to write the FPGA config word
@@ -80,40 +95,40 @@ void SetupSpi(int mode) {
         case SPI_FPGA_MODE:
             AT91C_BASE_SPI->SPI_MR =
                 (0 << 24)          |  // Delay between chip selects (take default: 6 MCK periods)
-                (0xE << 16)         | // Peripheral Chip Select (selects FPGA SPI_NCS0 or PA11)
+                (0xE << 16)        |  // Peripheral Chip Select (selects FPGA SPI_NCS0 or PA11)
                 (0 << 7)           |  // Local Loopback Disabled
-                AT91C_SPI_MODFDIS   | // Mode Fault Detection disabled
+                AT91C_SPI_MODFDIS  |  // Mode Fault Detection disabled
                 (0 << 2)           |  // Chip selects connected directly to peripheral
-                AT91C_SPI_PS_FIXED  | // Fixed Peripheral Select
+                AT91C_SPI_PS_FIXED |  // Fixed Peripheral Select
                 AT91C_SPI_MSTR;       // Master Mode
 
             AT91C_BASE_SPI->SPI_CSR[0] =
                 (1 << 24)          |  // Delay between Consecutive Transfers (32 MCK periods)
                 (1 << 16)          |  // Delay Before SPCK (1 MCK period)
                 (6 << 8)           |  // Serial Clock Baud Rate (baudrate = MCK/6 = 24MHz/6 = 4M baud
-                AT91C_SPI_BITS_16   | // Bits per Transfer (16 bits)
+                AT91C_SPI_BITS_16  |  // Bits per Transfer (16 bits)
                 (0 << 3)           |  // Chip Select inactive after transfer
-                AT91C_SPI_NCPHA     | // Clock Phase data captured on leading edge, changes on following edge
+                AT91C_SPI_NCPHA    |  // Clock Phase data captured on leading edge, changes on following edge
                 (0 << 0);             // Clock Polarity inactive state is logic 0
             break;
         /*
                     case SPI_LCD_MODE:
                     AT91C_BASE_SPI->SPI_MR =
-                        ( 0 << 24)          | // Delay between chip selects (take default: 6 MCK periods)
-                        (0xB << 16)         | // Peripheral Chip Select (selects LCD SPI_NCS2 or PA10)
-                        ( 0 << 7)           | // Local Loopback Disabled
-                        ( 1 << 4)           | // Mode Fault Detection disabled
-                        ( 0 << 2)           | // Chip selects connected directly to peripheral
-                        ( 0 << 1)           | // Fixed Peripheral Select
+                        ( 0 << 24)         |  // Delay between chip selects (take default: 6 MCK periods)
+                        (0xB << 16)        |  // Peripheral Chip Select (selects LCD SPI_NCS2 or PA10)
+                        ( 0 << 7)          |  // Local Loopback Disabled
+                        ( 1 << 4)          |  // Mode Fault Detection disabled
+                        ( 0 << 2)          |  // Chip selects connected directly to peripheral
+                        ( 0 << 1)          |  // Fixed Peripheral Select
                         ( 1 << 0);            // Master Mode
 
                     AT91C_BASE_SPI->SPI_CSR[2] =
-                        ( 1 << 24)          | // Delay between Consecutive Transfers (32 MCK periods)
-                        ( 1 << 16)          | // Delay Before SPCK (1 MCK period)
-                        ( 6 << 8)           | // Serial Clock Baud Rate (baudrate = MCK/6 = 24MHz/6 = 4M baud
-                        AT91C_SPI_BITS_9    | // Bits per Transfer (9 bits)
-                        ( 0 << 3)           | // Chip Select inactive after transfer
-                        ( 1 << 1)           | // Clock Phase data captured on leading edge, changes on following edge
+                        ( 1 << 24)         |  // Delay between Consecutive Transfers (32 MCK periods)
+                        ( 1 << 16)         |  // Delay Before SPCK (1 MCK period)
+                        ( 6 << 8)          |  // Serial Clock Baud Rate (baudrate = MCK/6 = 24MHz/6 = 4M baud
+                        AT91C_SPI_BITS_9   |  // Bits per Transfer (9 bits)
+                        ( 0 << 3)          |  // Chip Select inactive after transfer
+                        ( 1 << 1)          |  // Clock Phase data captured on leading edge, changes on following edge
                         ( 0 << 0);            // Clock Polarity inactive state is logic 0
                     break;
         */
@@ -127,7 +142,7 @@ void SetupSpi(int mode) {
 // Set up the synchronous serial port with the set of options that fits
 // the FPGA mode. Both RX and TX are always enabled.
 //-----------------------------------------------------------------------------
-void FpgaSetupSsc(void) {
+void FpgaSetupSsc(uint16_t fpga_mode) {
     // First configure the GPIOs, and get ourselves a clock.
     AT91C_BASE_PIOA->PIO_ASR =
         GPIO_SSC_FRAME  |
@@ -145,12 +160,17 @@ void FpgaSetupSsc(void) {
     // data and frame signal is sampled on falling edge of RK
     AT91C_BASE_SSC->SSC_RCMR = SSC_CLOCK_MODE_SELECT(1) | SSC_CLOCK_MODE_START(1);
 
-    // 8 bits per transfer, no loopback, MSB first, 1 transfer per sync
+    // 8, 16 or 32 bits per transfer, no loopback, MSB first, 1 transfer per sync
     // pulse, no output sync
-    AT91C_BASE_SSC->SSC_RFMR = SSC_FRAME_MODE_BITS_IN_WORD(8) | AT91C_SSC_MSBF | SSC_FRAME_MODE_WORDS_PER_TRANSFER(0);
+    if (((fpga_mode & FPGA_MAJOR_MODE_MASK) == FPGA_MAJOR_MODE_HF_READER) &&
+            (FpgaGetCurrent() == FPGA_BITSTREAM_HF || FpgaGetCurrent() == FPGA_BITSTREAM_HF_15)) {
+        AT91C_BASE_SSC->SSC_RFMR = SSC_FRAME_MODE_BITS_IN_WORD(16) | AT91C_SSC_MSBF | SSC_FRAME_MODE_WORDS_PER_TRANSFER(0);
+    } else {
+        AT91C_BASE_SSC->SSC_RFMR = SSC_FRAME_MODE_BITS_IN_WORD(8) | AT91C_SSC_MSBF | SSC_FRAME_MODE_WORDS_PER_TRANSFER(0);
+    }
 
-    // TX clock comes from TK pin, no clock output, outputs change on falling
-    // edge of TK, frame sync is sampled on rising edge of TK, start TX on rising edge of TF
+    // TX clock comes from TK pin, no clock output, outputs change on rising edge of TK,
+    // TF (frame sync) is sampled on falling edge of TK, start TX on rising edge of TF
     AT91C_BASE_SSC->SSC_TCMR = SSC_CLOCK_MODE_SELECT(2) | SSC_CLOCK_MODE_START(5);
 
     // tx framing is the same as the rx framing
@@ -164,8 +184,10 @@ void FpgaSetupSsc(void) {
 // a single buffer as a circular buffer (so that we just chain back to
 // ourselves, not to another buffer).
 //-----------------------------------------------------------------------------
-bool FpgaSetupSscDma(uint8_t *buf, int len) {
-    if (buf == NULL) return false;
+bool FpgaSetupSscDma(uint8_t *buf, uint16_t len) {
+    if (buf == NULL) {
+        return false;
+    }
 
     FpgaDisableSscDma();
     AT91C_BASE_PDC_SSC->PDC_RPR = (uint32_t) buf;  // transfer to this memory address
@@ -177,74 +199,81 @@ bool FpgaSetupSscDma(uint8_t *buf, int len) {
 }
 
 //----------------------------------------------------------------------------
-// Uncompress (inflate) the FPGA data. Returns one decompressed byte with
-// each call.
+// Uncompress (inflate) the FPGA data. Returns one decompressed byte with each call.
 //----------------------------------------------------------------------------
-static int get_from_fpga_combined_stream(z_streamp compressed_fpga_stream, uint8_t *output_buffer) {
-    if (fpga_image_ptr == compressed_fpga_stream->next_out) { // need more data
-        compressed_fpga_stream->next_out = output_buffer;
-        compressed_fpga_stream->avail_out = OUTPUT_BUFFER_LEN;
+static int get_from_fpga_combined_stream(lz4_streamp_t compressed_fpga_stream, uint8_t *output_buffer) {
+    if (fpga_image_ptr == output_buffer + FPGA_RING_BUFFER_BYTES) { // need more data
         fpga_image_ptr = output_buffer;
-        int res = inflate(compressed_fpga_stream, Z_SYNC_FLUSH);
-
-        if (res != Z_OK)
-            Dbprintf("inflate returned: %d, %s", res, compressed_fpga_stream->msg);
-
-        if (res < 0)
+        int cmp_bytes;
+        memcpy(&cmp_bytes, compressed_fpga_stream->next_in, sizeof(int));
+        compressed_fpga_stream->next_in += 4;
+        compressed_fpga_stream->avail_in -= cmp_bytes + 4;
+        int res = LZ4_decompress_safe_continue(compressed_fpga_stream->lz4StreamDecode,
+                                               compressed_fpga_stream->next_in,
+                                               (char *)output_buffer,
+                                               cmp_bytes,
+                                               FPGA_RING_BUFFER_BYTES);
+        if (res <= 0) {
+            Dbprintf("inflate returned: %d", res);
             return res;
+        }
+        compressed_fpga_stream->next_in += cmp_bytes;
     }
     uncompressed_bytes_cnt++;
     return *fpga_image_ptr++;
 }
 
+static int bitstream_target_to_index(FPGA_config bitstream_target) {
+    static int8_t bitstream_index_map[FPGA_CONFIG_COUNT] = {-1};
+
+    // Initialize
+    if (bitstream_index_map[FPGA_BITSTREAM_UNKNOWN] == -1) {
+        bitstream_index_map[FPGA_BITSTREAM_UNKNOWN] = 0;
+
+        for (size_t i = 0; i < g_fpga_bitstream_num; i++) {
+            FPGA_VERSION_INFORMATION info = g_fpga_version_information[i];
+            bitstream_index_map[info.target_config] = i;
+        }
+    }
+
+    return bitstream_index_map[bitstream_target];
+}
+
 //----------------------------------------------------------------------------
 // Undo the interleaving of several FPGA config files. FPGA config files
 // are combined into one big file:
 // 288 bytes from FPGA file 1, followed by 288 bytes from FGPA file 2, etc.
 //----------------------------------------------------------------------------
-static int get_from_fpga_stream(int bitstream_version, z_streamp compressed_fpga_stream, uint8_t *output_buffer) {
-    while ((uncompressed_bytes_cnt / FPGA_INTERLEAVE_SIZE) % g_fpga_bitstream_num != (bitstream_version - 1)) {
-        // skip undesired data belonging to other bitstream_versions
+static int get_from_fpga_stream(int bitstream_target, lz4_streamp_t compressed_fpga_stream, uint8_t *output_buffer) {
+    int bitstream_index = bitstream_target_to_index(bitstream_target);
+    while ((uncompressed_bytes_cnt / FPGA_INTERLEAVE_SIZE) % g_fpga_bitstream_num != bitstream_index) {
+        // skip undesired data belonging to other bitstream_targets
         get_from_fpga_combined_stream(compressed_fpga_stream, output_buffer);
     }
 
     return get_from_fpga_combined_stream(compressed_fpga_stream, output_buffer);
 }
 
-static voidpf fpga_inflate_malloc(voidpf opaque, uInt items, uInt size) {
-    return BigBuf_malloc(items * size);
-}
-
-// free eventually allocated BigBuf memory
-static void fpga_inflate_free(voidpf opaque, voidpf address) {
-    BigBuf_free();
-    BigBuf_Clear_ext(false);
-}
-
 //----------------------------------------------------------------------------
 // Initialize decompression of the respective (HF or LF) FPGA stream
 //----------------------------------------------------------------------------
-static bool reset_fpga_stream(int bitstream_version, z_streamp compressed_fpga_stream, uint8_t *output_buffer) {
+static bool reset_fpga_stream(int bitstream_target, lz4_streamp_t compressed_fpga_stream, uint8_t *output_buffer) {
     uint8_t header[FPGA_BITSTREAM_FIXED_HEADER_SIZE];
 
     uncompressed_bytes_cnt = 0;
 
     // initialize z_stream structure for inflate:
-    compressed_fpga_stream->next_in = &_binary_obj_fpga_all_bit_z_start;
-    compressed_fpga_stream->avail_in = &_binary_obj_fpga_all_bit_z_end - &_binary_obj_fpga_all_bit_z_start;
-    compressed_fpga_stream->next_out = output_buffer;
-    compressed_fpga_stream->avail_out = OUTPUT_BUFFER_LEN;
-    compressed_fpga_stream->zalloc = &fpga_inflate_malloc;
-    compressed_fpga_stream->zfree = &fpga_inflate_free;
+    compressed_fpga_stream->next_in = (char *)_binary_obj_fpga_all_bit_z_start;
+    compressed_fpga_stream->avail_in = (uint32_t)_binary_obj_fpga_all_bit_z_end - (uint32_t)_binary_obj_fpga_all_bit_z_start;
 
-    int res = inflateInit2(compressed_fpga_stream, 0);
-    if (res < 0)
+    int res = LZ4_setStreamDecode(compressed_fpga_stream->lz4StreamDecode, NULL, 0);
+    if (res == 0)
         return false;
 
-    fpga_image_ptr = output_buffer;
+    fpga_image_ptr = output_buffer + FPGA_RING_BUFFER_BYTES;
 
     for (uint16_t i = 0; i < FPGA_BITSTREAM_FIXED_HEADER_SIZE; i++)
-        header[i] = get_from_fpga_stream(bitstream_version, compressed_fpga_stream, output_buffer);
+        header[i] = get_from_fpga_stream(bitstream_target, compressed_fpga_stream, output_buffer);
 
     // Check for a valid .bit file (starts with bitparse_fixed_header)
     if (memcmp(bitparse_fixed_header, header, FPGA_BITSTREAM_FIXED_HEADER_SIZE) == 0)
@@ -266,12 +295,13 @@ static void DownloadFPGA_byte(uint8_t w) {
 }
 
 // Download the fpga image starting at current stream position with length FpgaImageLen bytes
-static void DownloadFPGA(int bitstream_version, int FpgaImageLen, z_streamp compressed_fpga_stream, uint8_t *output_buffer) {
+static void DownloadFPGA(int bitstream_target, int FpgaImageLen, lz4_streamp_t compressed_fpga_stream, uint8_t *output_buffer) {
     int i = 0;
-
+#if !defined XC3
     AT91C_BASE_PIOA->PIO_OER = GPIO_FPGA_ON;
     AT91C_BASE_PIOA->PIO_PER = GPIO_FPGA_ON;
     HIGH(GPIO_FPGA_ON);  // ensure everything is powered on
+#endif
 
     SpinDelay(50);
 
@@ -284,7 +314,13 @@ static void DownloadFPGA(int bitstream_version, int FpgaImageLen, z_streamp comp
     // PIO controls the following pins
     AT91C_BASE_PIOA->PIO_PER =
         GPIO_FPGA_NINIT |
+#if defined XC3
+        //3S100E M2 & M3 PIO ENA
+        GPIO_SPCK |
+        GPIO_MOSI |
+#endif
         GPIO_FPGA_DONE;
+
     // Enable pull-ups
     AT91C_BASE_PIOA->PIO_PPUER =
         GPIO_FPGA_NINIT |
@@ -298,8 +334,19 @@ static void DownloadFPGA(int bitstream_version, int FpgaImageLen, z_streamp comp
     AT91C_BASE_PIOA->PIO_OER =
         GPIO_FPGA_NPROGRAM |
         GPIO_FPGA_CCLK     |
+#if defined XC3
+        //3S100E M2 & M3 OUTPUT ENA
+        GPIO_SPCK |
+        GPIO_MOSI |
+#endif
         GPIO_FPGA_DIN;
 
+#if defined XC3
+    //3S100E M2 & M3 OUTPUT HIGH
+    HIGH(GPIO_SPCK);
+    HIGH(GPIO_MOSI);
+#endif
+
     // enter FPGA configuration mode
     LOW(GPIO_FPGA_NPROGRAM);
     SpinDelay(50);
@@ -318,8 +365,15 @@ static void DownloadFPGA(int bitstream_version, int FpgaImageLen, z_streamp comp
         return;
     }
 
+#if defined XC3
+    //3S100E M2 & M3 RETURN TO NORMAL
+    LOW(GPIO_SPCK);
+    LOW(GPIO_MOSI);
+    AT91C_BASE_PIOA->PIO_PDR = GPIO_SPCK | GPIO_MOSI;
+#endif
+
     for (i = 0; i < FpgaImageLen; i++) {
-        int b = get_from_fpga_stream(bitstream_version, compressed_fpga_stream, output_buffer);
+        int b = get_from_fpga_stream(bitstream_target, compressed_fpga_stream, output_buffer);
         if (b < 0) {
             Dbprintf("Error %d during FpgaDownload", b);
             break;
@@ -348,12 +402,14 @@ static void DownloadFPGA(int bitstream_version, int FpgaImageLen, z_streamp comp
  * (big endian), <length> bytes content. Except for section 'e' which has 4 bytes
  * length.
  */
-static int bitparse_find_section(int bitstream_version, char section_name, uint32_t *section_length, z_streamp compressed_fpga_stream, uint8_t *output_buffer) {
-    int result = 0;
+static int bitparse_find_section(int bitstream_target, char section_name, uint32_t *section_length, lz4_streamp_t compressed_fpga_stream, uint8_t *output_buffer) {
+
 #define MAX_FPGA_BIT_STREAM_HEADER_SEARCH 100  // maximum number of bytes to search for the requested section
+
+    int result = 0;
     uint16_t numbytes = 0;
     while (numbytes < MAX_FPGA_BIT_STREAM_HEADER_SEARCH) {
-        char current_name = get_from_fpga_stream(bitstream_version, compressed_fpga_stream, output_buffer);
+        char current_name = get_from_fpga_stream(bitstream_target, compressed_fpga_stream, output_buffer);
         numbytes++;
         uint32_t current_length = 0;
         if (current_name < 'a' || current_name > 'e') {
@@ -364,18 +420,24 @@ static int bitparse_find_section(int bitstream_version, char section_name, uint3
         switch (current_name) {
             case 'e':
                 /* Four byte length field */
-                current_length += get_from_fpga_stream(bitstream_version, compressed_fpga_stream, output_buffer) << 24;
-                current_length += get_from_fpga_stream(bitstream_version, compressed_fpga_stream, output_buffer) << 16;
+                current_length += get_from_fpga_stream(bitstream_target, compressed_fpga_stream, output_buffer) << 24;
+                current_length += get_from_fpga_stream(bitstream_target, compressed_fpga_stream, output_buffer) << 16;
+                current_length += get_from_fpga_stream(bitstream_target, compressed_fpga_stream, output_buffer) << 8;
+                current_length += get_from_fpga_stream(bitstream_target, compressed_fpga_stream, output_buffer) << 0;
+                numbytes += 4;
+                if (current_length > 300 * 1024) {
+                    /* section e should never exceed about 300KB, if the length is too big limit it but still send the bitstream just in case */
+                    current_length = 300 * 1024;
+                }
+                break;
+            default: /* Two byte length field */
+                current_length += get_from_fpga_stream(bitstream_target, compressed_fpga_stream, output_buffer) << 8;
+                current_length += get_from_fpga_stream(bitstream_target, compressed_fpga_stream, output_buffer) << 0;
                 numbytes += 2;
-            default: /* Fall through, two byte length field */
-                current_length += get_from_fpga_stream(bitstream_version, compressed_fpga_stream, output_buffer) << 8;
-                current_length += get_from_fpga_stream(bitstream_version, compressed_fpga_stream, output_buffer) << 0;
-                numbytes += 2;
-        }
-
-        if (current_name != 'e' && current_length > 255) {
-            /* Maybe a parse error */
-            break;
+                if (current_length > 64) {
+                    /* if text field is too long, keep it but truncate it */
+                    current_length = 64;
+                }
         }
 
         if (current_name == section_name) {
@@ -385,47 +447,98 @@ static int bitparse_find_section(int bitstream_version, char section_name, uint3
             break;
         }
 
-        for (uint16_t i = 0; i < current_length && numbytes < MAX_FPGA_BIT_STREAM_HEADER_SEARCH; i++) {
-            get_from_fpga_stream(bitstream_version, compressed_fpga_stream, output_buffer);
+        for (uint32_t i = 0; i < current_length && numbytes < MAX_FPGA_BIT_STREAM_HEADER_SEARCH; i++) {
+            get_from_fpga_stream(bitstream_target, compressed_fpga_stream, output_buffer);
             numbytes++;
         }
     }
     return result;
 }
 
+//----------------------------------------------------------------------------
+// Change FPGA image status, if image loaded.
+// bitstream_target is your new fpga image version
+// return true if can change.
+// return false if image is unloaded.
+//----------------------------------------------------------------------------
+#if defined XC3
+static bool FpgaConfCurrentMode(int bitstream_target) {
+    // fpga "XC3S100E" image merge
+    // If fpga image is no init
+    // We need load hf_lf_allinone.bit
+    if (downloaded_bitstream != FPGA_BITSTREAM_UNKNOWN) {
+        // test start
+        // PIO controls the following pins
+        AT91C_BASE_PIOA->PIO_PER = GPIO_FPGA_SWITCH;
+        // These pins are outputs
+        AT91C_BASE_PIOA->PIO_OER = GPIO_FPGA_SWITCH;
+
+        // try to turn off antenna
+        FpgaWriteConfWord(FPGA_MAJOR_MODE_OFF);
+
+        if (bitstream_target == FPGA_BITSTREAM_LF) {
+            LOW(GPIO_FPGA_SWITCH);
+        } else {
+            HIGH(GPIO_FPGA_SWITCH);
+        }
+        // update downloaded_bitstream
+        downloaded_bitstream = bitstream_target;
+        // turn off antenna
+        FpgaWriteConfWord(FPGA_MAJOR_MODE_OFF);
+        return true;
+    }
+    return false;
+}
+#endif
+
 //----------------------------------------------------------------------------
 // Check which FPGA image is currently loaded (if any). If necessary
 // decompress and load the correct (HF or LF) image to the FPGA
 //----------------------------------------------------------------------------
-void FpgaDownloadAndGo(int bitstream_version) {
+void FpgaDownloadAndGo(int bitstream_target) {
 
     // check whether or not the bitstream is already loaded
-    if (downloaded_bitstream == bitstream_version) {
+    if (downloaded_bitstream == bitstream_target) {
         FpgaEnableTracing();
         return;
     }
 
-    // Send waiting time extension request as this will take a while
-    send_wtx(1500);
-    z_stream compressed_fpga_stream;
-    uint8_t output_buffer[OUTPUT_BUFFER_LEN] = {0x00};
+#if defined XC3
+    // If we can change image version
+    // direct return.
+    if (FpgaConfCurrentMode(bitstream_target)) {
+        return;
+    }
+#endif
 
-    bool verbose = (DBGLEVEL > 3);
+    // Send waiting time extension request as this will take a while
+    send_wtx(FPGA_LOAD_WAIT_TIME);
+
+    bool verbose = (g_dbglevel > 3);
 
     // make sure that we have enough memory to decompress
     BigBuf_free();
     BigBuf_Clear_ext(verbose);
 
-    if (!reset_fpga_stream(bitstream_version, &compressed_fpga_stream, output_buffer))
+    lz4_stream_t compressed_fpga_stream;
+    LZ4_streamDecode_t lz4StreamDecode_body = {{ 0 }};
+    compressed_fpga_stream.lz4StreamDecode = &lz4StreamDecode_body;
+    uint8_t *output_buffer = BigBuf_malloc(FPGA_RING_BUFFER_BYTES);
+
+    if (!reset_fpga_stream(bitstream_target, &compressed_fpga_stream, output_buffer))
         return;
 
     uint32_t bitstream_length;
-    if (bitparse_find_section(bitstream_version, 'e', &bitstream_length, &compressed_fpga_stream, output_buffer)) {
-        DownloadFPGA(bitstream_version, bitstream_length, &compressed_fpga_stream, output_buffer);
-        downloaded_bitstream = bitstream_version;
+    if (bitparse_find_section(bitstream_target, 'e', &bitstream_length, &compressed_fpga_stream, output_buffer)) {
+        DownloadFPGA(bitstream_target, bitstream_length, &compressed_fpga_stream, output_buffer);
+        downloaded_bitstream = bitstream_target;
     }
 
-    inflateEnd(&compressed_fpga_stream);
+#if defined XC3
+    // first download fpga image to hf
+    // we need to change fpga status to hf
+    FpgaConfCurrentMode(bitstream_target);
+#endif
 
     // turn off antenna
     FpgaWriteConfWord(FPGA_MAJOR_MODE_OFF);
@@ -440,7 +553,7 @@ void FpgaDownloadAndGo(int bitstream_version) {
 // The bit format is:  C3 C2 C1 C0 D11 D10 D9 D8 D7 D6 D5 D4 D3 D2 D1 D0
 // where C is the 4 bit command and D is the 12 bit data
 //
-// @params cmd and v  gets or over eachother.  Take careful note of overlapping bits.
+// @params cmd and v  gets OR:ED over each other.  Take careful note of overlapping bits.
 //-----------------------------------------------------------------------------
 void FpgaSendCommand(uint16_t cmd, uint16_t v) {
     SetupSpi(SPI_FPGA_MODE);
@@ -451,7 +564,7 @@ void FpgaSendCommand(uint16_t cmd, uint16_t v) {
 //-----------------------------------------------------------------------------
 // Write the FPGA setup word (that determines what mode the logic is in, read
 // vs. clone vs. etc.). This is now a special case of FpgaSendCommand() to
-// avoid changing this function's occurence everywhere in the source code.
+// avoid changing this function's occurrence everywhere in the source code.
 //-----------------------------------------------------------------------------
 void FpgaWriteConfWord(uint16_t v) {
     FpgaSendCommand(FPGA_CMD_SET_CONFREG, v);
@@ -508,8 +621,8 @@ void SetAdcMuxFor(uint32_t whichGpio) {
 }
 
 void Fpga_print_status(void) {
-    DbpString(_BLUE_("Currently loaded FPGA image"));
-    Dbprintf("  mode....................%s", g_fpga_version_information[downloaded_bitstream - 1]);
+    DbpString(_CYAN_("Current FPGA image"));
+    Dbprintf("  mode.................... %s", g_fpga_version_information[bitstream_target_to_index(downloaded_bitstream)]);
 }
 
 int FpgaGetCurrent(void) {
@@ -521,10 +634,15 @@ int FpgaGetCurrent(void) {
 // if HF,  Disable SSC DMA
 // turn off trace and leds off.
 void switch_off(void) {
-    if (DBGLEVEL > 3) Dbprintf("switch_off");
+    if (g_dbglevel > 3) {
+        Dbprintf("switch_off");
+    }
+
     FpgaWriteConfWord(FPGA_MAJOR_MODE_OFF);
-    if (downloaded_bitstream == FPGA_BITSTREAM_HF)
+    if (downloaded_bitstream == FPGA_BITSTREAM_HF || downloaded_bitstream == FPGA_BITSTREAM_HF_15) {
         FpgaDisableSscDma();
+    }
+
     set_tracing(false);
     LEDsoff();
 }
