@@ -1,11 +1,17 @@
 //-----------------------------------------------------------------------------
-// (c) 2009 Henryk Plötz <henryk@ploetzli.ch>
-//     2016 Iceman
-//     2018 AntiCat
+// Copyright (C) Proxmark3 contributors. See AUTHORS.md for details.
 //
-// This code is licensed to you under the terms of the GNU GPL, version 2 or,
-// at your option, any later version. See the LICENSE.txt file for the text of
-// the license.
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// See LICENSE.txt for the text of the license.
 //-----------------------------------------------------------------------------
 // LEGIC RF simulation code
 //-----------------------------------------------------------------------------
@@ -38,10 +44,10 @@ static crc_t legic_crc;
 //  - 100us for a 1 (RWD_TIME_1)
 //
 // The data dependent timing makes writing comprehensible code significantly
-// harder. The current aproach forwards the prng data based if there is data on
+// harder. The current approach forwards the prng data based if there is data on
 // air and time based, using GET_TICKS, during computational and wait periodes.
 //
-// To not have the necessity to calculate/guess exection time dependend timeouts
+// To not have the necessity to calculate/guess execution time dependent timeouts
 // tx_frame and rx_frame use a shared timestamp to coordinate tx and rx timeslots.
 //-----------------------------------------------------------------------------
 
@@ -58,66 +64,72 @@ static uint32_t last_frame_end; /* ts of last bit of previews rx or tx frame */
 #define LEGIC_CARD_MEMSIZE 1024 /* The largest Legic Prime card is 1k */
 #define WRITE_LOWERLIMIT      4 /* UID and MCC are not writable */
 
-#define INPUT_THRESHOLD       8 /* heuristically determined, lower values */
+static uint32_t input_threshold = 8; /* heuristically determined, lower values */
 /* lead to detecting false ack during write */
 
 //-----------------------------------------------------------------------------
 // I/O interface abstraction (FPGA -> ARM)
 //-----------------------------------------------------------------------------
-static uint8_t rx_byte_from_fpga(void) {
+static uint16_t rx_frame_from_fpga(void) {
     for (;;) {
         WDT_HIT();
 
-        // wait for byte be become available in rx holding register
+        // wait for frame be become available in rx holding register
         if (AT91C_BASE_SSC->SSC_SR & (AT91C_SSC_RXRDY)) {
             return AT91C_BASE_SSC->SSC_RHR;
         }
     }
+    return 0;
 }
 
 //-----------------------------------------------------------------------------
-// Demodulation
+// Demodulation (Reader)
 //-----------------------------------------------------------------------------
 
-// Returns am aproximated power measurement
+// Returns a demodulated bit
 //
-// The FPGA running on the xcorrelation kernel samples the subcarrier at ~3 MHz.
-// The kernel was initialy designed to receive BSPK/2-PSK. Hance, it reports an
-// I/Q pair every 18.9us (8 bits i and 8 bits q).
+// The FPGA running xcorrelation samples the subcarrier at ~13.56 MHz. The mode
+// was initially designed to receive BSPK/2-PSK. Hance, it reports an I/Q pair
+// every 4.7us (8 bits i and 8 bits q).
 //
 // The subcarrier amplitude can be calculated using Pythagoras sqrt(i^2 + q^2).
 // To reduce CPU time the amplitude is approximated by using linear functions:
 //   am = MAX(ABS(i),ABS(q)) + 1/2*MIN(ABS(i),ABSq))
 //
-// Note: The SSC receiver is never synchronized the calculation may be performed
-// on a i/q pair from two subsequent correlations, but does not matter.
-// Note: inlining this function would fail with -Os
-static int32_t sample_power(void) {
-    int32_t q = (int8_t)rx_byte_from_fpga();
-    q = ABS(q);
-    int32_t i = (int8_t)rx_byte_from_fpga();
-    i = ABS(i);
-
-    return MAX(i, q) + (MIN(i, q) >> 1);
-}
-
-// Returns a demedulated bit
+// The bit time is 99.1us (21 I/Q pairs). The receiver skips the first 5 samples
+// and averages the next (most stable) 8 samples. The final 8 samples are dropped
+// also.
 //
-// An aproximated power measurement is available every 18.9us. The bit time
-// is 100us. The code samples 5 times and uses the last (most stable) sample.
+// The demodulated should be aligned to the bit period by the caller. This is
+// done in rx_bit and rx_ack.
 //
 // Note: The demodulator would be drifting (18.9us * 5 != 100us), rx_frame
 // has a delay loop that aligns rx_bit calls to the TAG tx timeslots.
-
+//
 // Note: inlining this function would fail with -Os
 static bool rx_bit(void) {
-    int32_t power;
+    int32_t sum_cq = 0;
+    int32_t sum_ci = 0;
 
+    // skip first 5 I/Q pairs
     for (size_t i = 0; i < 5; ++i) {
-        power = sample_power();
+        (void)rx_frame_from_fpga();
     }
 
-    return (power > INPUT_THRESHOLD);
+    // sample next 8 I/Q pairs
+    for (uint8_t i = 0; i < 8; ++i) {
+        uint16_t iq = rx_frame_from_fpga();
+        int8_t ci = (int8_t)(iq >> 8);
+        int8_t cq = (int8_t)(iq & 0xff);
+        sum_ci += ci;
+        sum_cq += cq;
+    }
+
+    // calculate power
+    int32_t power = (MAX(ABS(sum_ci), ABS(sum_cq)) + (MIN(ABS(sum_ci), ABS(sum_cq)) >> 1));
+
+    // compare average (power / 8) to threshold
+    return ((power >> 3) > input_threshold);
 }
 
 //-----------------------------------------------------------------------------
@@ -131,18 +143,18 @@ static bool rx_bit(void) {
 
 static void tx_bit(bool bit) {
     // insert pause
-    LOW(GPIO_SSC_DOUT);
+    HIGH(GPIO_SSC_DOUT);
     last_frame_end += RWD_TIME_PAUSE;
     while (GET_TICKS < last_frame_end) { };
-    HIGH(GPIO_SSC_DOUT);
 
-    // return to high, wait for bit periode to end
+    // return to carrier on, wait for bit periode to end
+    LOW(GPIO_SSC_DOUT);
     last_frame_end += (bit ? RWD_TIME_1 : RWD_TIME_0) - RWD_TIME_PAUSE;
     while (GET_TICKS < last_frame_end) { };
 }
 
 //-----------------------------------------------------------------------------
-// Frame Handling
+// Frame Handling (Reader)
 //
 // The LEGIC RF protocol from card to reader does not include explicit frame
 // start/stop information or length information. The reader must know beforehand
@@ -152,7 +164,7 @@ static void tx_bit(bool bit) {
 //-----------------------------------------------------------------------------
 
 static void tx_frame(uint32_t frame, uint8_t len) {
-    FpgaWriteConfWord(FPGA_MAJOR_MODE_HF_READER_TX);
+    FpgaWriteConfWord(FPGA_MAJOR_MODE_HF_READER | FPGA_HF_READER_MODE_SEND_FULL_MOD);
 
     // wait for next tx timeslot
     last_frame_end += RWD_FRAME_WAIT;
@@ -169,10 +181,10 @@ static void tx_frame(uint32_t frame, uint8_t len) {
     };
 
     // add pause to mark end of the frame
-    LOW(GPIO_SSC_DOUT);
+    HIGH(GPIO_SSC_DOUT);
     last_frame_end += RWD_TIME_PAUSE;
     while (GET_TICKS < last_frame_end) { };
-    HIGH(GPIO_SSC_DOUT);
+    LOW(GPIO_SSC_DOUT);
 
     // log
     uint8_t cmdbytes[] = {len, BYTEx(frame, 0), BYTEx(frame, 1), BYTEx(frame, 2)};
@@ -180,9 +192,7 @@ static void tx_frame(uint32_t frame, uint8_t len) {
 }
 
 static uint32_t rx_frame(uint8_t len) {
-    FpgaWriteConfWord(FPGA_MAJOR_MODE_HF_READER_RX_XCORR
-                      | FPGA_HF_READER_RX_XCORR_848_KHZ
-                      | FPGA_HF_READER_RX_XCORR_QUARTER);
+    FpgaWriteConfWord(FPGA_MAJOR_MODE_HF_READER | FPGA_HF_READER_SUBCARRIER_212_KHZ | FPGA_HF_READER_MODE_RECEIVE_IQ);
 
     // hold sampling until card is expected to respond
     last_frame_end += TAG_FRAME_WAIT;
@@ -210,9 +220,7 @@ static uint32_t rx_frame(uint8_t len) {
 
 static bool rx_ack(void) {
     // change fpga into rx mode
-    FpgaWriteConfWord(FPGA_MAJOR_MODE_HF_READER_RX_XCORR
-                      | FPGA_HF_READER_RX_XCORR_848_KHZ
-                      | FPGA_HF_READER_RX_XCORR_QUARTER);
+    FpgaWriteConfWord(FPGA_MAJOR_MODE_HF_READER | FPGA_HF_READER_SUBCARRIER_212_KHZ | FPGA_HF_READER_MODE_RECEIVE_IQ);
 
     // hold sampling until card is expected to respond
     last_frame_end += TAG_FRAME_WAIT;
@@ -271,27 +279,25 @@ static int init_card(uint8_t cardtype, legic_card_select_t *p_card) {
             p_card->cmdsize = 0;
             p_card->addrsize = 0;
             p_card->cardsize = 0;
-            return 2;
+            return PM3_ESOFT;
     }
-    return 0;
+    return PM3_SUCCESS;
 }
 
-static void init_reader(bool clear_mem) {
+static void init_reader(void) {
     // configure FPGA
     FpgaDownloadAndGo(FPGA_BITSTREAM_HF);
-    FpgaWriteConfWord(FPGA_MAJOR_MODE_HF_READER_RX_XCORR
-                      | FPGA_HF_READER_RX_XCORR_848_KHZ
-                      | FPGA_HF_READER_RX_XCORR_QUARTER);
+    FpgaWriteConfWord(FPGA_MAJOR_MODE_HF_READER | FPGA_HF_READER_SUBCARRIER_212_KHZ | FPGA_HF_READER_MODE_RECEIVE_IQ);
     SetAdcMuxFor(GPIO_MUXSEL_HIPKD);
     LED_A_ON();
 
     // configure SSC with defaults
-    FpgaSetupSsc();
+    FpgaSetupSsc(FPGA_MAJOR_MODE_HF_READER);
 
     // re-claim GPIO_SSC_DOUT as GPIO and enable output
     AT91C_BASE_PIOA->PIO_OER = GPIO_SSC_DOUT;
     AT91C_BASE_PIOA->PIO_PER = GPIO_SSC_DOUT;
-    HIGH(GPIO_SSC_DOUT);
+    LOW(GPIO_SSC_DOUT);
 
     // reserve a cardmem, meaning we can use the tracelog function in bigbuff easier.
     legic_mem = BigBuf_get_EM_addr();
@@ -412,12 +418,12 @@ legic_card_select_t *getLegicCardInfo(void) {
 
 void LegicRfInfo(void) {
     // configure ARM and FPGA
-    init_reader(false);
+    init_reader();
 
     // establish shared secret and detect card type
     uint8_t card_type = setup_phase(0x01);
-    if (init_card(card_type, &card) != 0) {
-        reply_mix(CMD_ACK, 0, 0, 0, 0, 0);
+    if (init_card(card_type, &card) != PM3_SUCCESS) {
+        reply_ng(CMD_HF_LEGIC_INFO, PM3_EINIT, NULL, 0);
         goto OUT;
     }
 
@@ -425,7 +431,7 @@ void LegicRfInfo(void) {
     for (uint8_t i = 0; i < sizeof(card.uid); ++i) {
         int16_t byte = read_byte(i, card.cmdsize);
         if (byte == -1) {
-            reply_mix(CMD_ACK, 0, 0, 0, 0, 0);
+            reply_ng(CMD_HF_LEGIC_INFO, PM3_EFAILED, NULL, 0);
             goto OUT;
         }
         card.uid[i] = byte & 0xFF;
@@ -435,12 +441,12 @@ void LegicRfInfo(void) {
     int16_t mcc = read_byte(4, card.cmdsize);
     int16_t calc_mcc = CRC8Legic(card.uid, 4);
     if (mcc != calc_mcc) {
-        reply_mix(CMD_ACK, 0, 0, 0, 0, 0);
+        reply_ng(CMD_HF_LEGIC_INFO, PM3_ESOFT, NULL, 0);
         goto OUT;
     }
 
     // OK
-    reply_mix(CMD_ACK, 1, 0, 0, (uint8_t *)&card, sizeof(legic_card_select_t));
+    reply_ng(CMD_HF_LEGIC_INFO, PM3_SUCCESS, (uint8_t *)&card, sizeof(legic_card_select_t));
 
 OUT:
     switch_off();
@@ -452,11 +458,11 @@ int LegicRfReaderEx(uint16_t offset, uint16_t len, uint8_t iv) {
     int res = PM3_SUCCESS;
 
     // configure ARM and FPGA
-    init_reader(false);
+    init_reader();
 
     // establish shared secret and detect card type
     uint8_t card_type = setup_phase(iv);
-    if (init_card(card_type, &card) != 0) {
+    if (init_card(card_type, &card) != PM3_SUCCESS) {
         res = PM3_ESOFT;
         goto OUT;
     }
@@ -487,12 +493,12 @@ OUT:
 
 void LegicRfReader(uint16_t offset, uint16_t len, uint8_t iv) {
     // configure ARM and FPGA
-    init_reader(false);
+    init_reader();
 
     // establish shared secret and detect card type
     uint8_t card_type = setup_phase(iv);
-    if (init_card(card_type, &card) != 0) {
-        reply_mix(CMD_ACK, 0, 0, 0, 0, 0);
+    if (init_card(card_type, &card) != PM3_SUCCESS) {
+        reply_ng(CMD_HF_LEGIC_READER, PM3_EINIT, NULL, 0);
         goto OUT;
     }
 
@@ -504,7 +510,7 @@ void LegicRfReader(uint16_t offset, uint16_t len, uint8_t iv) {
     for (uint16_t i = 0; i < len; ++i) {
         int16_t byte = read_byte(offset + i, card.cmdsize);
         if (byte == -1) {
-            reply_mix(CMD_ACK, 0, 0, 0, 0, 0);
+            reply_ng(CMD_HF_LEGIC_READER, PM3_EFAILED, NULL, 0);
             goto OUT;
         }
         legic_mem[i] = byte;
@@ -515,27 +521,27 @@ void LegicRfReader(uint16_t offset, uint16_t len, uint8_t iv) {
     }
 
     // OK
-    reply_mix(CMD_ACK, 1, len, 0, 0, 0);
+    reply_ng(CMD_HF_LEGIC_READER, PM3_SUCCESS, (uint8_t *)&len, sizeof(len));
 
 OUT:
     switch_off();
     StopTicks();
 }
 
-void LegicRfWriter(uint16_t offset, uint16_t len, uint8_t iv, uint8_t *data) {
+void LegicRfWriter(uint16_t offset, uint16_t len, uint8_t iv, const uint8_t *data) {
     // configure ARM and FPGA
-    init_reader(false);
+    init_reader();
 
     // uid is not writeable
     if (offset <= WRITE_LOWERLIMIT) {
-        reply_mix(CMD_ACK, 0, 0, 0, 0, 0);
+        reply_ng(CMD_HF_LEGIC_WRITER, PM3_EINVARG, NULL, 0);
         goto OUT;
     }
 
     // establish shared secret and detect card type
     uint8_t card_type = setup_phase(iv);
-    if (init_card(card_type, &card) != 0) {
-        reply_mix(CMD_ACK, 0, 0, 0, 0, 0);
+    if (init_card(card_type, &card) != PM3_SUCCESS) {
+        reply_ng(CMD_HF_LEGIC_WRITER, PM3_EINIT, NULL, 0);
         goto OUT;
     }
 
@@ -545,18 +551,22 @@ void LegicRfWriter(uint16_t offset, uint16_t len, uint8_t iv, uint8_t *data) {
     }
 
     // write in reverse order, only then is DCF (decremental field) writable
-    while (len-- > 0 && !BUTTON_PRESS()) {
-        if (!write_byte(len + offset, data[len], card.addrsize)) {
+    while (len-- > 0 && BUTTON_PRESS() == false) {
+        if (write_byte(len + offset, data[len], card.addrsize) == false) {
             Dbprintf("operation failed | %02X | %02X | %02X", len + offset, len, data[len]);
-            reply_mix(CMD_ACK, 0, 0, 0, 0, 0);
+            reply_ng(CMD_HF_LEGIC_WRITER, PM3_EFAILED, NULL, 0);
             goto OUT;
         }
     }
 
     // OK
-    reply_mix(CMD_ACK, 1, len, 0, 0, 0);
+    reply_ng(CMD_HF_LEGIC_WRITER, PM3_SUCCESS, (uint8_t *)&len, sizeof(len));
 
 OUT:
     switch_off();
     StopTicks();
 }
+
+void LegicRfSetThreshold(uint32_t threshold) {
+    input_threshold = threshold;
+}
