@@ -1,7 +1,17 @@
 //-----------------------------------------------------------------------------
-// This code is licensed to you under the terms of the GNU GPL, version 2 or,
-// at your option, any later version. See the LICENSE.txt file for the text of
-// the license.
+// Copyright (C) Proxmark3 contributors. See AUTHORS.md for details.
+//
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// See LICENSE.txt for the text of the license.
 //-----------------------------------------------------------------------------
 // LF ADC read/write implementation
 //-----------------------------------------------------------------------------
@@ -17,6 +27,7 @@
 
 extern bool g_logging;
 
+uint8_t get_adc_avg(void);
 void lf_sample_mean(void);
 bool lf_test_periods(size_t expected, size_t count);
 size_t lf_count_edge_periods(size_t max);
@@ -28,10 +39,11 @@ bool lf_get_reader_modulation(void);
 
 void lf_wait_periods(size_t periods);
 //void lf_init(bool reader);
-void lf_init(bool reader, bool simulate);
-void lf_finalize(void);
+void lf_init(bool reader, bool simulate, bool ledcontrol);
+void lf_finalize(bool ledcontrol);
 size_t lf_detect_field_drop(size_t max);
-bool lf_manchester_send_bytes(const uint8_t *frame, size_t frame_len);
+
+bool lf_manchester_send_bytes(const uint8_t *frame, size_t frame_len, bool ledcontrol);
 void lf_modulation(bool modulation);
 
 #endif // __LFADC_H__
