@@ -1,7 +1,17 @@
 //-----------------------------------------------------------------------------
-// This code is licensed to you under the terms of the GNU GPL, version 2 or,
-// at your option, any later version. See the LICENSE.txt file for the text of
-// the license.
+// Copyright (C) Proxmark3 contributors. See AUTHORS.md for details.
+//
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// See LICENSE.txt for the text of the license.
 //-----------------------------------------------------------------------------
 // Miscellaneous routines for low frequency sampling.
 //-----------------------------------------------------------------------------
@@ -16,6 +26,8 @@
 #include "util.h"
 #include "lfdemod.h"
 #include "string.h"  // memset
+#include "appmain.h" // print stack
+#include "usb_cdc.h" // real-time sampling
 
 /*
 Default LF config is set to:
@@ -27,17 +39,49 @@ Default LF config is set to:
     samples_to_skip = 0
     verbose = YES
     */
-static sample_config config = { 1, 8, 1, LF_DIVISOR_125, 0, 0, 1} ;
 
-void printConfig(void) {
+static const sample_config def_config = {
+    .decimation = 1,
+    .bits_per_sample = 8,
+    .averaging = 1,
+    .divisor = LF_DIVISOR_125,
+    .trigger_threshold = 0,
+    .samples_to_skip = 0,
+    .verbose = false,
+};
+
+static sample_config config = { 1, 8, 1, LF_DIVISOR_125, 0, 0, true} ;
+
+// Holds bit packed struct of samples.
+static BitstreamOut_t data = {0, 0, 0};
+
+// internal struct to keep track of samples gathered
+static sampling_t samples = {0, 0, 0, 0};
+
+void printLFConfig(void) {
     uint32_t d = config.divisor;
-    DbpString(_BLUE_("LF Sampling config"));
-    Dbprintf("  [q] divisor.............%d ( "_GREEN_("%d.%02d kHz")")", d, 12000 / (d + 1), ((1200000 + (d + 1) / 2) / (d + 1)) - ((12000 / (d + 1)) * 100));
-    Dbprintf("  [b] bits per sample.....%d", config.bits_per_sample);
-    Dbprintf("  [d] decimation..........%d", config.decimation);
-    Dbprintf("  [a] averaging...........%s", (config.averaging) ? "Yes" : "No");
-    Dbprintf("  [t] trigger threshold...%d", config.trigger_threshold);
-    Dbprintf("  [s] samples to skip.....%d ", config.samples_to_skip);
+    DbpString(_CYAN_("LF Sampling config"));
+    Dbprintf("  [q] divisor............. %d ( "_GREEN_("%d.%02d kHz")" )", d, 12000 / (d + 1), ((1200000 + (d + 1) / 2) / (d + 1)) - ((12000 / (d + 1)) * 100));
+    Dbprintf("  [b] bits per sample..... %d", config.bits_per_sample);
+    Dbprintf("  [d] decimation.......... %d", config.decimation);
+    Dbprintf("  [a] averaging........... %s", (config.averaging) ? "yes" : "no");
+    Dbprintf("  [t] trigger threshold... %d", config.trigger_threshold);
+    Dbprintf("  [s] samples to skip..... %d ", config.samples_to_skip);
+    DbpString("");
+}
+
+void printSamples(void) {
+    DbpString(_CYAN_("LF Sampling memory usage"));
+//    Dbprintf("  decimation counter...%d", samples.dec_counter);
+//    Dbprintf("  sum..................%u", samples.sum);
+    Dbprintf("  counter.............. " _YELLOW_("%u"), samples.counter);
+    Dbprintf("  total saved.......... " _YELLOW_("%u"), samples.total_saved);
+    print_stack_usage();
+}
+
+
+void setDefaultSamplingConfig(void) {
+    setSamplingConfig(&def_config);
 }
 
 /**
@@ -51,14 +95,14 @@ void printConfig(void) {
  * @brief setSamplingConfig
  * @param sc
  */
-void setSamplingConfig(sample_config *sc) {
+void setSamplingConfig(const sample_config *sc) {
 
     // decimation (1-8) how many bits of adc sample value to save
-    if (sc->decimation > 0 && sc->decimation < 8)
+    if (sc->decimation > 0 && sc->decimation < 9)
         config.decimation = sc->decimation;
 
     // bits per sample (1-8)
-    if (sc->bits_per_sample > 0 && sc->bits_per_sample < 8)
+    if (sc->bits_per_sample > 0 && sc->bits_per_sample < 9)
         config.bits_per_sample = sc->bits_per_sample;
 
     //
@@ -78,47 +122,26 @@ void setSamplingConfig(sample_config *sc) {
         config.samples_to_skip = sc->samples_to_skip;
 
     if (sc->verbose)
-        printConfig();
+        printLFConfig();
 }
 
 sample_config *getSamplingConfig(void) {
     return &config;
 }
 
-/**
- * @brief Pushes bit onto the stream
- * @param stream
- * @param bit
- */
-static void pushBit(BitstreamOut *stream, uint8_t bit) {
-    int bytepos = stream->position >> 3; // divide by 8
-    int bitpos = stream->position & 7;
-    *(stream->buffer + bytepos) &= ~(1 << (7 - bitpos));
-    *(stream->buffer + bytepos) |= (bit > 0) << (7 - bitpos);
-    stream->position++;
-    stream->numbits++;
-}
-
-// Holds bit packed struct of samples.
-static BitstreamOut data = {0, 0, 0};
-
-// internal struct to keep track of samples gathered
-static sampling_t samples = {0, 0, 0, 0};
-
 void initSampleBuffer(uint32_t *sample_size) {
     initSampleBufferEx(sample_size, false);
 }
 
 void initSampleBufferEx(uint32_t *sample_size, bool use_malloc) {
+
     if (sample_size == NULL) {
-        Dbprintf("initSampleBufferEx, param NULL");
         return;
     }
-    BigBuf_free();
 
+    BigBuf_free_keep_EM();
 
     // We can't erase the buffer now, it would drastically delay the acquisition
-
     if (use_malloc) {
 
         if (*sample_size == 0) {
@@ -132,14 +155,20 @@ void initSampleBufferEx(uint32_t *sample_size, bool use_malloc) {
     } else {
         if (*sample_size == 0) {
             *sample_size = BigBuf_max_traceLen();
+        } else {
+            *sample_size = MIN(*sample_size, BigBuf_max_traceLen());
         }
         data.buffer = BigBuf_get_addr();
     }
 
-    //
+    // reset data stream
+    data.numbits = 0;
+    data.position = 0;
+
+    // reset samples
     samples.dec_counter = 0;
     samples.sum = 0;
-    samples.counter = 0;
+    samples.counter = *sample_size;
     samples.total_saved = 0;
 }
 
@@ -153,14 +182,26 @@ void logSampleSimple(uint8_t sample) {
 
 void logSample(uint8_t sample, uint8_t decimation, uint8_t bits_per_sample, bool avg) {
 
-    if (!data.buffer) return;
-
-    if (bits_per_sample == 0) bits_per_sample = 1;
-    if (bits_per_sample > 8) bits_per_sample = 8;
-    if (decimation == 0) decimation = 1;
+    if (!data.buffer) {
+        return;
+    }
 
     // keep track of total gather samples regardless how many was discarded.
-    samples.counter++;
+    if (samples.counter-- == 0) {
+        return;
+    }
+
+    if (bits_per_sample == 0) {
+        bits_per_sample = 1;
+    }
+
+    if (bits_per_sample > 8) {
+        bits_per_sample = 8;
+    }
+
+    if (decimation == 0) {
+        decimation = 1;
+    }
 
     if (avg) {
         samples.sum += sample;
@@ -170,7 +211,9 @@ void logSample(uint8_t sample, uint8_t decimation, uint8_t bits_per_sample, bool
     if (decimation > 1) {
         samples.dec_counter++;
 
-        if (samples.dec_counter < decimation) return;
+        if (samples.dec_counter < decimation) {
+            return;
+        }
 
         samples.dec_counter = 0;
     }
@@ -192,13 +235,20 @@ void logSample(uint8_t sample, uint8_t decimation, uint8_t bits_per_sample, bool
         data.numbits = samples.total_saved << 3;
 
     } else {
-        pushBit(&data, sample & 0x80);
-        if (bits_per_sample > 1) pushBit(&data, sample & 0x40);
-        if (bits_per_sample > 2) pushBit(&data, sample & 0x20);
-        if (bits_per_sample > 3) pushBit(&data, sample & 0x10);
-        if (bits_per_sample > 4) pushBit(&data, sample & 0x08);
-        if (bits_per_sample > 5) pushBit(&data, sample & 0x04);
-        if (bits_per_sample > 6) pushBit(&data, sample & 0x02);
+        // truncate trailing data
+        sample >>= 8 - bits_per_sample;
+        sample <<= 8 - bits_per_sample;
+
+        uint8_t bits_offset = data.numbits & 0x7;
+        uint8_t bits_cap = 8 - bits_offset;
+
+        // write the current byte
+        data.buffer[data.numbits >> 3] |= sample >> bits_offset;
+        uint32_t numbits = data.numbits + bits_cap;
+
+        // write the remaining bits to the next byte
+        data.buffer[numbits >> 3] |= sample << (bits_cap);
+        data.numbits += bits_per_sample;
     }
 }
 
@@ -211,26 +261,31 @@ void logSample(uint8_t sample, uint8_t decimation, uint8_t bits_per_sample, bool
 **/
 void LFSetupFPGAForADC(int divisor, bool reader_field) {
     FpgaDownloadAndGo(FPGA_BITSTREAM_LF);
-    if ((divisor == 1) || (divisor < 0) || (divisor > 255))
+    if ((divisor == 1) || (divisor < 0) || (divisor > 255)) {
         FpgaSendCommand(FPGA_CMD_SET_DIVISOR, LF_DIVISOR_134); //~134kHz
-    else if (divisor == 0)
+    } else if (divisor == 0) {
         FpgaSendCommand(FPGA_CMD_SET_DIVISOR, LF_DIVISOR_125); //125kHz
-    else
+    } else {
         FpgaSendCommand(FPGA_CMD_SET_DIVISOR, divisor);
+    }
 
     FpgaWriteConfWord(FPGA_MAJOR_MODE_LF_READER | (reader_field ? FPGA_LF_ADC_READER_FIELD : 0));
 
     // Connect the A/D to the peak-detected low-frequency path.
     SetAdcMuxFor(GPIO_MUXSEL_LOPKD);
-    // 50ms for the resonant antenna to settle.
-    if (reader_field)
-        SpinDelay(50);
 
     // Now set up the SSC to get the ADC samples that are now streaming at us.
-    FpgaSetupSsc();
+    FpgaSetupSsc(FPGA_MAJOR_MODE_LF_READER);
 
     // start a 1.5ticks is 1us
     StartTicks();
+
+    // 50ms for the resonant antenna to settle.
+    if (reader_field) {
+        WaitMS(50);
+    } else {
+        WaitMS(1);
+    }
 }
 
 /**
@@ -249,18 +304,25 @@ void LFSetupFPGAForADC(int divisor, bool reader_field) {
  * @return the number of bits occupied by the samples.
  */
 uint32_t DoAcquisition(uint8_t decimation, uint8_t bits_per_sample, bool avg, int16_t trigger_threshold,
-                       bool verbose, uint32_t sample_size, uint32_t cancel_after, int32_t samples_to_skip) {
+                       bool verbose, uint32_t sample_size, uint32_t cancel_after, int32_t samples_to_skip, bool ledcontrol) {
 
-    initSampleBuffer(&sample_size);
+    initSampleBuffer(&sample_size); // sample size in bytes
+    sample_size <<= 3; // sample size in bits
+    sample_size /= bits_per_sample; // sample count
 
+    if (g_dbglevel >= DBG_DEBUG) {
+        printSamples();
+    }
+
+    bool trigger_hit = false;
     uint32_t cancel_counter = 0;
     int16_t checked = 0;
 
-    while (!BUTTON_PRESS()) {
+    while (BUTTON_PRESS() == false) {
 
-        // only every 1000th times, in order to save time when collecting samples.
+        // only every 4000th times, in order to save time when collecting samples.
         // interruptible only when logging not yet triggered
-        if ((checked == 1000) && (trigger_threshold > 0)) {
+        if (trigger_hit == false && (checked >= 4000)) {
             if (data_available()) {
                 checked = -1;
                 break;
@@ -272,29 +334,29 @@ uint32_t DoAcquisition(uint8_t decimation, uint8_t bits_per_sample, bool avg, in
 
         WDT_HIT();
 
-        if (AT91C_BASE_SSC->SSC_SR & AT91C_SSC_TXRDY) {
-//			AT91C_BASE_SSC->SSC_THR = 0x43;
+        if (ledcontrol && (AT91C_BASE_SSC->SSC_SR & AT91C_SSC_TXRDY)) {
             LED_D_ON();
         }
 
         if (AT91C_BASE_SSC->SSC_SR & AT91C_SSC_RXRDY) {
             volatile uint8_t sample = (uint8_t)AT91C_BASE_SSC->SSC_RHR;
 
-            // Testpoint 8 (TP8) can be used to trigger oscilliscope
-            LED_D_OFF();
+            // (RDV4) Test point 8 (TP8) can be used to trigger oscilloscope
+            if (ledcontrol) LED_D_OFF();
 
             // threshold either high or low values 128 = center 0.  if trigger = 178
-            if ((trigger_threshold > 0) && (sample < (trigger_threshold + 128)) && (sample > (128 - trigger_threshold))) {
-                if (cancel_after > 0) {
-                    cancel_counter++;
-                    if (cancel_after == cancel_counter)
-                        break;
+            if (trigger_hit == false) {
+                if ((trigger_threshold > 0) && (sample < (trigger_threshold + 128)) && (sample > (128 - trigger_threshold))) {
+                    if (cancel_after > 0) {
+                        cancel_counter++;
+                        if (cancel_after == cancel_counter)
+                            break;
+                    }
+                    continue;
                 }
-                continue;
+                trigger_hit = true;
             }
 
-            trigger_threshold = 0;
-
             if (samples_to_skip > 0) {
                 samples_to_skip--;
                 continue;
@@ -306,20 +368,25 @@ uint32_t DoAcquisition(uint8_t decimation, uint8_t bits_per_sample, bool avg, in
         }
     }
 
-    if (checked == -1 && verbose) {
-        Dbprintf("lf sampling aborted");
-    }
-
     if (verbose) {
+        if (checked == -1) {
+            Dbprintf("lf sampling aborted");
+        } else if ((cancel_counter == cancel_after) && (cancel_after > 0)) {
+            Dbprintf("lf sampling cancelled after %u", cancel_counter);
+        }
+
         Dbprintf("Done, saved " _YELLOW_("%d")" out of " _YELLOW_("%d")" seen samples at " _YELLOW_("%d")" bits/sample", samples.total_saved, samples.counter, bits_per_sample);
     }
 
     // Ensure that DC offset removal and noise check is performed for any device-side processing
-    removeSignalOffset(data.buffer, samples.total_saved);
-    computeSignalProperties(data.buffer, samples.total_saved);
-
+    if (bits_per_sample == 8) {
+        // these functions only consider bps==8
+        removeSignalOffset(data.buffer, samples.total_saved);
+        computeSignalProperties(data.buffer, samples.total_saved);
+    }
     return data.numbits;
 }
+
 /**
  * @brief Does sample acquisition, ignoring the config values set in the sample_config.
  * This method is typically used by tag-specific readers who just wants to read the samples
@@ -328,30 +395,40 @@ uint32_t DoAcquisition(uint8_t decimation, uint8_t bits_per_sample, bool avg, in
  * @param verbose
  * @return number of bits sampled
  */
-uint32_t DoAcquisition_default(int trigger_threshold, bool verbose) {
-    return DoAcquisition(1, 8, 0, trigger_threshold, verbose, 0, 0, 0);
+uint32_t DoAcquisition_default(int trigger_threshold, bool verbose, bool ledcontrol) {
+    return DoAcquisition(1, 8, 0, trigger_threshold, verbose, 0, 0, 0, ledcontrol);
 }
-uint32_t DoAcquisition_config(bool verbose, uint32_t sample_size) {
+uint32_t DoAcquisition_config(bool verbose, uint32_t sample_size, bool ledcontrol) {
     return DoAcquisition(config.decimation
                          , config.bits_per_sample
                          , config.averaging
                          , config.trigger_threshold
                          , verbose
                          , sample_size
+                         , 0    // cancel_after
+                         , config.samples_to_skip
+                         , ledcontrol);
+}
+
+uint32_t DoPartialAcquisition(int trigger_threshold, bool verbose, uint32_t sample_size, uint32_t cancel_after, bool ledcontrol) {
+    return DoAcquisition(config.decimation
+                         , config.bits_per_sample
+                         , config.averaging
+                         , trigger_threshold
+                         , verbose
+                         , sample_size
+                         , cancel_after
                          , 0
-                         , config.samples_to_skip);
+                         , ledcontrol);  // samples to skip
 }
 
-uint32_t DoPartialAcquisition(int trigger_threshold, bool verbose, uint32_t sample_size, uint32_t cancel_after) {
-    return DoAcquisition(1, 8, 0, trigger_threshold, verbose, sample_size, cancel_after, 0);
-}
-
-static uint32_t ReadLF(bool reader_field, bool verbose, uint32_t sample_size) {
+static uint32_t ReadLF(bool reader_field, bool verbose, uint32_t sample_size, bool ledcontrol) {
     if (verbose)
-        printConfig();
+        printLFConfig();
 
     LFSetupFPGAForADC(config.divisor, reader_field);
-    uint32_t ret = DoAcquisition_config(verbose, sample_size);
+    uint32_t ret = DoAcquisition_config(verbose, sample_size, ledcontrol);
+    StopTicks();
     FpgaWriteConfWord(FPGA_MAJOR_MODE_OFF);
     return ret;
 }
@@ -360,24 +437,144 @@ static uint32_t ReadLF(bool reader_field, bool verbose, uint32_t sample_size) {
 * Initializes the FPGA for reader-mode (field on), and acquires the samples.
 * @return number of bits sampled
 **/
-uint32_t SampleLF(bool verbose, uint32_t sample_size) {
+uint32_t SampleLF(bool verbose, uint32_t sample_size, bool ledcontrol) {
     BigBuf_Clear_ext(false);
-    return ReadLF(true, verbose, sample_size);
+    return ReadLF(true, verbose, sample_size, ledcontrol);
+}
+
+/**
+ * Do LF sampling and send samples to the USB
+ *
+ * Uses parameters in config. Only bits_per_sample = 8 is working now
+ *
+ * @param reader_field - true for reading tags, false for sniffing
+ * @return sampling result
+**/
+int ReadLF_realtime(bool reader_field) {
+    // parameters from config and constants
+    const uint8_t bits_per_sample = config.bits_per_sample;
+    const int16_t trigger_threshold = config.trigger_threshold;
+    int32_t samples_to_skip = config.samples_to_skip;
+    const uint8_t decimation = config.decimation;
+
+    const int8_t size_threshold_table[9] = {0, 64, 64, 60, 64, 60, 60, 56, 64};
+    const int8_t size_threshold = size_threshold_table[bits_per_sample];
+
+    // DoAcquisition() start
+    uint8_t last_byte = 0;
+    uint8_t curr_byte = 0;
+    int return_value = PM3_SUCCESS;
+
+    uint32_t sample_buffer_len = AT91C_USB_EP_IN_SIZE;
+    initSampleBuffer(&sample_buffer_len);
+    if (sample_buffer_len != AT91C_USB_EP_IN_SIZE) {
+        return PM3_EFAILED;
+    }
+
+    bool trigger_hit = false;
+    int16_t checked = 0;
+
+    return_value = async_usb_write_start();
+    if (return_value != PM3_SUCCESS) {
+        return return_value;
+    }
+
+    BigBuf_Clear_ext(false);
+    LFSetupFPGAForADC(config.divisor, reader_field);
+
+    while (BUTTON_PRESS() == false) {
+        // only every 4000th times, in order to save time when collecting samples.
+        // interruptible only when logging not yet triggered
+        if (trigger_hit == false && (checked >= 4000)) {
+            if (data_available()) {
+                checked = -1;
+                break;
+            } else {
+                checked = 0;
+            }
+        }
+        ++checked;
+
+        WDT_HIT();
+
+        if ((AT91C_BASE_SSC->SSC_SR & AT91C_SSC_TXRDY)) {
+            LED_D_ON();
+        }
+
+        if (AT91C_BASE_SSC->SSC_SR & AT91C_SSC_RXRDY) {
+            volatile uint8_t sample = (uint8_t)AT91C_BASE_SSC->SSC_RHR;
+
+            // (RDV4) Test point 8 (TP8) can be used to trigger oscilloscope
+            LED_D_OFF();
+
+            // threshold either high or low values 128 = center 0.  if trigger = 178
+            if (trigger_hit == false) {
+                if ((trigger_threshold > 0) && (sample < (trigger_threshold + 128)) && (sample > (128 - trigger_threshold))) {
+                    continue;
+                }
+                trigger_hit = true;
+            }
+
+            if (samples_to_skip > 0) {
+                samples_to_skip--;
+                continue;
+            }
+
+            logSample(sample, decimation, bits_per_sample, false);
+
+            // Write to USB FIFO if byte changed
+            curr_byte = data.numbits >> 3;
+            if (curr_byte > last_byte) {
+                async_usb_write_pushByte(data.buffer[last_byte]);
+            }
+            last_byte = curr_byte;
+
+            if (samples.total_saved == size_threshold) {
+                // Request USB transmission and change FIFO bank
+                if (async_usb_write_requestWrite() == false) {
+                    return_value = PM3_EIO;
+                    goto out;
+                }
+
+                // Reset sample
+                last_byte = 0;
+                data.numbits = 0;
+                samples.counter = size_threshold;
+                samples.total_saved = 0;
+
+            } else if (samples.total_saved == 1) {
+                // Check if there is any data from client
+                if (data_available_fast()) {
+                    break;
+                }
+            }
+        }
+    }
+
+    return_value = async_usb_write_stop();
+
+out:
+    LED_D_OFF();
+
+    // DoAcquisition() end
+    StopTicks();
+    FpgaWriteConfWord(FPGA_MAJOR_MODE_OFF);
+    return return_value;
 }
 /**
 * Initializes the FPGA for sniffer-mode (field off), and acquires the samples.
 * @return number of bits sampled
 **/
-uint32_t SniffLF(void) {
+uint32_t SniffLF(bool verbose, uint32_t sample_size, bool ledcontrol) {
     BigBuf_Clear_ext(false);
-    return ReadLF(false, true, 0);
+    return ReadLF(false, verbose, sample_size, ledcontrol);
 }
 
 /**
 * acquisition of T55x7 LF signal. Similar to other LF, but adjusted with @marshmellows thresholds
 * the data is collected in BigBuf.
 **/
-void doT55x7Acquisition(size_t sample_size) {
+void doT55x7Acquisition(size_t sample_size, bool ledcontrol) {
 
 #define T55xx_READ_UPPER_THRESHOLD 128+60  // 60 grph
 #define T55xx_READ_LOWER_THRESHOLD 128-60  // -60 grph
@@ -397,9 +594,18 @@ void doT55x7Acquisition(size_t sample_size) {
 
     uint16_t checker = 0;
 
+    if (g_dbglevel >= DBG_DEBUG) {
+        Dbprintf("doT55x7Acquisition - after init");
+        print_stack_usage();
+    }
+
     while (skipCnt < 1000 && (i < bufsize)) {
-        if (checker == 1000) {
-            if (BUTTON_PRESS() || data_available())
+
+        if (BUTTON_PRESS())
+            break;
+
+        if (checker == 4000) {
+            if (data_available())
                 break;
             else
                 checker = 0;
@@ -409,13 +615,13 @@ void doT55x7Acquisition(size_t sample_size) {
 
         WDT_HIT();
 
-        if (AT91C_BASE_SSC->SSC_SR & AT91C_SSC_TXRDY) {
+        if (ledcontrol && (AT91C_BASE_SSC->SSC_SR & AT91C_SSC_TXRDY)) {
             LED_D_ON();
         }
 
         if (AT91C_BASE_SSC->SSC_SR & AT91C_SSC_RXRDY) {
             volatile uint8_t sample = (uint8_t)AT91C_BASE_SSC->SSC_RHR;
-            LED_D_OFF();
+            if (ledcontrol) LED_D_OFF();
 
             // skip until the first high sample above threshold
             if (!startFound && sample > T55xx_READ_UPPER_THRESHOLD) {
@@ -436,12 +642,14 @@ void doT55x7Acquisition(size_t sample_size) {
             // skip until first high samples begin to change
             if (startFound || sample > T55xx_READ_LOWER_THRESHOLD + T55xx_READ_TOL) {
                 // if just found start - recover last sample
-                if (!startFound) {
+                if (startFound == false) {
                     dest[i++] = lastSample;
                     startFound = true;
                 }
                 // collect samples
-                dest[i++] = sample;
+                if (i < bufsize) {
+                    dest[i++] = sample;
+                }
             }
         }
     }
@@ -452,30 +660,31 @@ void doT55x7Acquisition(size_t sample_size) {
 **/
 
 #define COTAG_T1 384
-#define COTAG_T2 (COTAG_T1>>1)
-#define COTAG_ONE_THRESHOLD 128+10
-#define COTAG_ZERO_THRESHOLD 128-10
+#define COTAG_T2 (COTAG_T1 >> 1)
+#define COTAG_ONE_THRESHOLD 127+5
+#define COTAG_ZERO_THRESHOLD 127-5
 #ifndef COTAG_BITS
 #define COTAG_BITS 264
 #endif
-void doCotagAcquisition(size_t sample_size) {
+void doCotagAcquisition(void) {
 
-    uint8_t *dest = BigBuf_get_addr();
     uint16_t bufsize = BigBuf_max_traceLen();
-
-    if (bufsize > sample_size)
-        bufsize = sample_size;
+    uint8_t *dest = BigBuf_malloc(bufsize);
 
     dest[0] = 0;
-    uint8_t firsthigh = 0, firstlow = 0;
-    uint16_t i = 0;
-    uint16_t noise_counter = 0;
+
+    bool firsthigh = false, firstlow = false;
+    uint16_t i = 0, noise_counter = 0;
 
     uint16_t checker = 0;
 
-    while ((i < bufsize) && (noise_counter < (COTAG_T1 << 1))) {
-        if (checker == 1000) {
-            if (BUTTON_PRESS() || data_available())
+    while ((i < bufsize - 1) && (noise_counter < COTAG_T1 << 1)) {
+
+        if (BUTTON_PRESS())
+            break;
+
+        if (checker == 4000) {
+            if (data_available())
                 break;
             else
                 checker = 0;
@@ -485,65 +694,68 @@ void doCotagAcquisition(size_t sample_size) {
 
         WDT_HIT();
 
-        if (AT91C_BASE_SSC->SSC_SR & AT91C_SSC_TXRDY) {
-            LED_D_ON();
-        }
-
         if (AT91C_BASE_SSC->SSC_SR & AT91C_SSC_RXRDY) {
+
             volatile uint8_t sample = (uint8_t)AT91C_BASE_SSC->SSC_RHR;
 
             // find first peak
-            if (!firsthigh) {
+            if (firsthigh == false) {
                 if (sample < COTAG_ONE_THRESHOLD) {
                     noise_counter++;
                     continue;
                 }
+
                 noise_counter = 0;
-                firsthigh = 1;
+                firsthigh = true;
             }
-            if (!firstlow) {
+
+            if (firstlow == false) {
                 if (sample > COTAG_ZERO_THRESHOLD) {
                     noise_counter++;
                     continue;
                 }
+
                 noise_counter = 0;
-                firstlow = 1;
+                firstlow = true;
             }
 
-            ++i;
-
-            if (sample > COTAG_ONE_THRESHOLD)
+            if (sample > COTAG_ONE_THRESHOLD) {
                 dest[i] = 255;
-            else if (sample < COTAG_ZERO_THRESHOLD)
+                ++i;
+            } else if (sample < COTAG_ZERO_THRESHOLD) {
                 dest[i] = 0;
-            else
+                ++i;
+            } else {
                 dest[i] = dest[i - 1];
+                ++i;
+            }
         }
     }
 
     // Ensure that DC offset removal and noise check is performed for any device-side processing
-    removeSignalOffset(dest, bufsize);
-    computeSignalProperties(dest, bufsize);
+    removeSignalOffset(dest, i);
+    computeSignalProperties(dest, i);
 }
 
-uint32_t doCotagAcquisitionManchester(void) {
+uint16_t doCotagAcquisitionManchester(uint8_t *dest, uint16_t destlen) {
 
-    uint8_t *dest = BigBuf_get_addr();
-    uint16_t bufsize = BigBuf_max_traceLen();
-
-    if (bufsize > COTAG_BITS)
-        bufsize = COTAG_BITS;
+    if (dest == NULL) {
+        return 0;
+    }
 
     dest[0] = 0;
-    uint8_t firsthigh = 0, firstlow = 0;
-    uint16_t sample_counter = 0, period = 0;
-    uint8_t curr = 0, prev = 0;
-    uint16_t noise_counter = 0;
-    uint16_t checker = 0;
 
-    while ((sample_counter < bufsize) && (noise_counter < (COTAG_T1 << 1))) {
-        if (checker == 1000) {
-            if (BUTTON_PRESS() || data_available())
+    bool firsthigh = false, firstlow = false;
+    uint8_t curr = 0, prev = 0;
+    uint16_t i = 0;
+    uint16_t period = 0, checker = 0;
+
+    while ((i < destlen) && BUTTON_PRESS() == false) {
+
+        WDT_HIT();
+
+        if (checker == 4000) {
+            if (data_available())
                 break;
             else
                 checker = 0;
@@ -551,32 +763,23 @@ uint32_t doCotagAcquisitionManchester(void) {
             ++checker;
         }
 
-        WDT_HIT();
-
-        if (AT91C_BASE_SSC->SSC_SR & AT91C_SSC_TXRDY) {
-            LED_D_ON();
-        }
 
         if (AT91C_BASE_SSC->SSC_SR & AT91C_SSC_RXRDY) {
             volatile uint8_t sample = (uint8_t)AT91C_BASE_SSC->SSC_RHR;
 
             // find first peak
-            if (!firsthigh) {
+            if (firsthigh == false) {
                 if (sample < COTAG_ONE_THRESHOLD) {
-                    noise_counter++;
                     continue;
                 }
-                noise_counter = 0;
-                firsthigh = 1;
+                firsthigh = true;
             }
 
-            if (!firstlow) {
+            if (firstlow == false) {
                 if (sample > COTAG_ZERO_THRESHOLD) {
-                    noise_counter++;
                     continue;
                 }
-                noise_counter = 0;
-                firstlow = 1;
+                firstlow = true;
             }
 
             // set sample 255, 0,  or previous
@@ -596,10 +799,11 @@ uint32_t doCotagAcquisitionManchester(void) {
                 continue;
             }
 
-            dest[sample_counter] = curr;
-            ++sample_counter;
+            dest[i] = curr;
+            ++i;
             period = COTAG_T1;
         }
     }
-    return sample_counter;
+
+    return i;
 }
