@@ -1,3 +1,18 @@
+//-----------------------------------------------------------------------------
+// Copyright (C) Proxmark3 contributors. See AUTHORS.md for details.
+//
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// See LICENSE.txt for the text of the license.
+//-----------------------------------------------------------------------------
 #include "mifaredesfire.h"
 
 #include "common.h"
@@ -46,7 +61,7 @@ bool InitDesfireCard(void) {
     set_tracing(true);
 
     if (!iso14443a_select_card(NULL, &card, NULL, true, 0, false)) {
-        if (DBGLEVEL >= DBG_ERROR) DbpString("Can't select card");
+        if (g_dbglevel >= DBG_ERROR) DbpString("Can't select card");
         OnError(1);
         return false;
     }
@@ -69,7 +84,7 @@ void MifareSendCommand(uint8_t *datain) {
     uint8_t resp[RECEIVE_SIZE];
     memset(resp, 0, sizeof(resp));
 
-    if (DBGLEVEL >= DBG_EXTENDED) {
+    if (g_dbglevel >= DBG_EXTENDED) {
         Dbprintf(" flags : %02X", payload->flags);
         Dbprintf(" len   : %02X", payload->datalen);
         print_result(" RX    : ", payload->datain, payload->datalen);
@@ -85,7 +100,7 @@ void MifareSendCommand(uint8_t *datain) {
     }
 
     int len = DesfireAPDU(payload->datain, payload->datalen, resp);
-    if (DBGLEVEL >= DBG_EXTENDED)
+    if (g_dbglevel >= DBG_EXTENDED)
         print_result("RESP <--: ", resp, len);
 
     if (!len) {
@@ -118,11 +133,13 @@ void MifareDesfireGetInformation(void) {
     struct p {
         uint8_t isOK;
         uint8_t uid[7];
+        uint8_t uidlen;
         uint8_t versionHW[7];
         uint8_t versionSW[7];
         uint8_t details[14];
     } PACKED payload;
 
+    memset(&payload, 0x00, sizeof(payload));
     /*
         1 = PCB                 1
         2 = cid                 2
@@ -141,22 +158,18 @@ void MifareDesfireGetInformation(void) {
 
     // card select - information
     if (!iso14443a_select_card(NULL, &card, NULL, true, 0, false)) {
-        if (DBGLEVEL >= DBG_ERROR) DbpString("Can't select card");
+        if (g_dbglevel >= DBG_ERROR) {
+            DbpString("Can't select card");
+        }
         payload.isOK = 1;  // 2 == can not select
         reply_ng(CMD_HF_DESFIRE_INFO, PM3_ESOFT, (uint8_t *)&payload, sizeof(payload));
         switch_off();
         return;
     }
 
-    if (card.uidlen != 7) {
-        if (DBGLEVEL >= DBG_ERROR) Dbprintf("Wrong UID size. Expected 7byte got %d", card.uidlen);
-        payload.isOK = 2;  // 2 == WRONG UID
-        reply_ng(CMD_HF_DESFIRE_INFO, PM3_ESOFT, (uint8_t *)&payload, sizeof(payload));
-        switch_off();
-        return;
-    }
     // add uid.
-    memcpy(payload.uid, card.uid, sizeof(payload.uid));
+    memcpy(payload.uid, card.uid, card.uidlen);
+    payload.uidlen = card.uidlen;
 
     LED_A_ON();
     uint8_t cmd[] = {0x90, MFDES_GET_VERSION, 0x00, 0x00, 0x00};
@@ -165,41 +178,71 @@ void MifareDesfireGetInformation(void) {
     len =  DesfireAPDU(cmd, cmd_len, resp);
     if (!len) {
         print_result("ERROR <--: ", resp, len);
-        payload.isOK = 3;  // 3 == DOESNT ANSWER TO GET_VERSION
+        payload.isOK = 3;  // 3 == DOESN'T ANSWER TO GET_VERSION
         reply_ng(CMD_HF_DESFIRE_INFO, PM3_ESOFT, (uint8_t *)&payload, sizeof(payload));
         switch_off();
         return;
     }
 
+    if (len < sizeof(payload.versionHW) + 1) {
+        payload.isOK = 3;  // 3 == DOESN'T ANSWER TO GET_VERSION
+        reply_ng(CMD_HF_DESFIRE_INFO, PM3_ESOFT, (uint8_t *)&payload, sizeof(payload));
+        switch_off();
+        if (g_dbglevel >= DBG_ERROR) {
+            Dbprintf("Tag answer to MFDES_GET_VERSION was too short: data in Hardware Information is probably invalid.");
+            print_result("Answer", resp, len);
+        }
+        return;
+    }
+
     memcpy(payload.versionHW, resp + 1, sizeof(payload.versionHW));
 
     // ADDITION_FRAME 1
     cmd[1] = MFDES_ADDITIONAL_FRAME;
-    len =  DesfireAPDU(cmd, cmd_len, resp);
+    len = DesfireAPDU(cmd, cmd_len, resp);
     if (!len) {
-        print_result("ERROR <--: ", resp, len);
-        payload.isOK = 3;  // 3 == DOESNT ANSWER TO GET_VERSION
+        payload.isOK = 3;  // 3 == DOESN'T ANSWER TO GET_VERSION
         reply_ng(CMD_HF_DESFIRE_INFO, PM3_ESOFT, (uint8_t *)&payload, sizeof(payload));
         switch_off();
         return;
     }
+
+    if (len < sizeof(payload.versionSW) + 1) {
+        payload.isOK = 3;  // 3 == DOESN'T ANSWER TO GET_VERSION
+        reply_ng(CMD_HF_DESFIRE_INFO, PM3_ESOFT, (uint8_t *)&payload, sizeof(payload));
+        switch_off();
+        if (g_dbglevel >= DBG_ERROR) {
+            Dbprintf("Tag answer to MFDES_ADDITIONAL_FRAME 1 was too short: data in Software Information is probably invalid.");
+            print_result("Answer", resp, len);
+        }
+        return;
+    }
+
     memcpy(payload.versionSW, resp + 1,  sizeof(payload.versionSW));
 
     // ADDITION_FRAME 2
     len =  DesfireAPDU(cmd, cmd_len, resp);
     if (!len) {
-        print_result("ERROR <--: ", resp, len);
-        payload.isOK = 3;  // 3 == DOESNT ANSWER TO GET_VERSION
+        payload.isOK = 3;  // 3 == DOESN'T ANSWER TO GET_VERSION
         reply_ng(CMD_HF_DESFIRE_INFO, PM3_ESOFT, (uint8_t *)&payload, sizeof(payload));
         switch_off();
         return;
     }
 
+    if (len < sizeof(payload.details) + 1) {
+        payload.isOK = 3;  // 3 == DOESN'T ANSWER TO GET_VERSION
+        reply_ng(CMD_HF_DESFIRE_INFO, PM3_ESOFT, (uint8_t *)&payload, sizeof(payload));
+        switch_off();
+        if (g_dbglevel >= DBG_ERROR) {
+            Dbprintf("Tag answer to MFDES_ADDITIONAL_FRAME 2 was too short: data in Batch number and Production date is probably invalid");
+            print_result("Answer", resp, len);
+        }
+        return;
+    }
+
     memcpy(payload.details, resp + 1,  sizeof(payload.details));
 
-    LED_B_ON();
     reply_ng(CMD_HF_DESFIRE_INFO, PM3_SUCCESS, (uint8_t *)&payload, sizeof(payload));
-    LED_B_OFF();
 
     // reset the pcb_blocknum,
     pcb_blocknum = 0;
@@ -276,7 +319,7 @@ void MifareDES_Auth1(uint8_t *datain) {
             memcpy(keybytes, PICC_MASTER_KEY8, 8);
         } else if (payload->algo == MFDES_ALGO_AES || payload->algo == MFDES_ALGO_3DES) {
             memcpy(keybytes, PICC_MASTER_KEY16, 16);
-        } else if (payload->algo == MFDES_ALGO_3DES) {
+        } else if (payload->algo == MFDES_ALGO_3K3DES) {
             memcpy(keybytes, PICC_MASTER_KEY24, 24);
         }
     } else {
@@ -300,10 +343,11 @@ void MifareDES_Auth1(uint8_t *datain) {
 
     uint8_t subcommand = MFDES_AUTHENTICATE;
 
-    if (payload->mode == MFDES_AUTH_AES)
+    if (payload->mode == MFDES_AUTH_AES) {
         subcommand = MFDES_AUTHENTICATE_AES;
-    else if (payload->mode == MFDES_AUTH_ISO)
+    } else if (payload->mode == MFDES_AUTH_ISO) {
         subcommand = MFDES_AUTHENTICATE_ISO;
+    }
 
     if (payload->mode != MFDES_AUTH_PICC) {
         // Let's send our auth command
@@ -321,8 +365,8 @@ void MifareDES_Auth1(uint8_t *datain) {
         len = DesfireAPDU(cmd, 2, resp);
     }
 
-    if (!len) {
-        if (DBGLEVEL >= DBG_ERROR) {
+    if (len == 0) {
+        if (g_dbglevel >= DBG_ERROR) {
             DbpString("Authentication failed. Card timeout.");
         }
         OnErrorNG(CMD_HF_DESFIRE_AUTH1, 3);
@@ -348,7 +392,7 @@ void MifareDES_Auth1(uint8_t *datain) {
     }
 
     if (len != expectedlen) {
-        if (DBGLEVEL >= DBG_ERROR) {
+        if (g_dbglevel >= DBG_ERROR) {
             DbpString("Authentication failed. Length of answer doesn't match algo.");
             print_result("Res-Buffer: ", resp, len);
         }
@@ -365,20 +409,23 @@ void MifareDES_Auth1(uint8_t *datain) {
 
     // Part 3
     if (payload->algo == MFDES_ALGO_AES) {
+
         if (mbedtls_aes_setkey_dec(&ctx, key->data, 128) != 0) {
-            if (DBGLEVEL >= DBG_EXTENDED) {
+            if (g_dbglevel >= DBG_EXTENDED) {
                 DbpString("mbedtls_aes_setkey_dec failed");
             }
             OnErrorNG(CMD_HF_DESFIRE_AUTH1, 7);
             return;
         }
         mbedtls_aes_crypt_cbc(&ctx, MBEDTLS_AES_DECRYPT, 16, IV, encRndB, RndB);
-    } else if (payload->algo == MFDES_ALGO_DES)
+
+    } else if (payload->algo == MFDES_ALGO_DES) {
         des_decrypt(RndB, encRndB, key->data);
-    else if (payload->algo == MFDES_ALGO_3DES)
+    } else if (payload->algo == MFDES_ALGO_3DES) {
         tdes_nxp_receive(encRndB, RndB, rndlen, key->data, IV, 2);
-    else if (payload->algo == MFDES_ALGO_3K3DES)
+    } else if (payload->algo == MFDES_ALGO_3K3DES) {
         tdes_nxp_receive(encRndB, RndB, rndlen, key->data, IV, 3);
+    }
 
     // - Rotate RndB by 8 bits
     memcpy(rotRndB, RndB, rndlen);
@@ -388,6 +435,7 @@ void MifareDES_Auth1(uint8_t *datain) {
 
     // - Encrypt our response
     if (payload->mode == MFDES_AUTH_DES || payload->mode == MFDES_AUTH_PICC) {
+
         des_decrypt(encRndA, RndA, key->data);
         memcpy(both, encRndA, rndlen);
 
@@ -397,7 +445,9 @@ void MifareDES_Auth1(uint8_t *datain) {
 
         des_decrypt(encRndB, rotRndB, key->data);
         memcpy(both + 8, encRndB, rndlen);
+
     } else if (payload->mode == MFDES_AUTH_ISO) {
+
         if (payload->algo == MFDES_ALGO_3DES) {
             uint8_t tmp[16] = {0x00};
             memcpy(tmp, RndA, rndlen);
@@ -409,13 +459,17 @@ void MifareDES_Auth1(uint8_t *datain) {
             memcpy(tmp + rndlen, rotRndB, rndlen);
             tdes_nxp_send(tmp, both, 32, key->data, IV, 3);
         }
+
     } else if (payload->mode == MFDES_AUTH_AES) {
+
         uint8_t tmp[32] = {0x00};
         memcpy(tmp, RndA, rndlen);
         memcpy(tmp + 16, rotRndB, rndlen);
+
         if (payload->algo == MFDES_ALGO_AES) {
+
             if (mbedtls_aes_setkey_enc(&ctx, key->data, 128) != 0) {
-                if (DBGLEVEL >= DBG_EXTENDED) {
+                if (g_dbglevel >= DBG_EXTENDED) {
                     DbpString("mbedtls_aes_setkey_enc failed");
                 }
                 OnErrorNG(CMD_HF_DESFIRE_AUTH1, 7);
@@ -429,6 +483,7 @@ void MifareDES_Auth1(uint8_t *datain) {
     if (payload->algo == MFDES_ALGO_AES || payload->algo == MFDES_ALGO_3K3DES) {
         bothlen = 32;
     }
+
     if (payload->mode != MFDES_AUTH_PICC) {
         cmd[0] = 0x90;
         cmd[1] = MFDES_ADDITIONAL_FRAME;
@@ -445,7 +500,7 @@ void MifareDES_Auth1(uint8_t *datain) {
     }
 
     if (!len) {
-        if (DBGLEVEL >= DBG_ERROR) {
+        if (g_dbglevel >= DBG_ERROR) {
             DbpString("Authentication failed. Card timeout.");
         }
         OnErrorNG(CMD_HF_DESFIRE_AUTH1, 3);
@@ -453,25 +508,30 @@ void MifareDES_Auth1(uint8_t *datain) {
     }
 
     if (payload->mode != MFDES_AUTH_PICC) {
+
         if ((resp[len - 4] != 0x91) || (resp[len - 3] != 0x00)) {
             DbpString("Authentication failed.");
             OnErrorNG(CMD_HF_DESFIRE_AUTH1, 6);
             return;
         }
+
     } else {
+
         if (resp[1] != 0x00) {
             DbpString("Authentication failed.");
             OnErrorNG(CMD_HF_DESFIRE_AUTH1, 6);
             return;
         }
+
     }
 
     // Part 4
 
     Desfire_session_key_new(RndA, RndB, key, sessionkey);
 
-    if (DBGLEVEL >= DBG_EXTENDED)
+    if (g_dbglevel >= DBG_EXTENDED) {
         print_result("SESSIONKEY : ", sessionkey->data, payload->keylen);
+    }
 
     if (payload->mode != MFDES_AUTH_PICC) {
         memcpy(encRndA, resp + 1, rndlen);
@@ -480,15 +540,19 @@ void MifareDES_Auth1(uint8_t *datain) {
     }
 
     if (payload->mode == MFDES_AUTH_DES || payload->mode == MFDES_AUTH_PICC) {
-        if (payload->algo == MFDES_ALGO_DES)
+
+        if (payload->algo == MFDES_ALGO_DES) {
             des_decrypt(encRndA, encRndA, key->data);
-        else if (payload->algo == MFDES_ALGO_3DES)
+        } else if (payload->algo == MFDES_ALGO_3DES) {
             tdes_nxp_receive(encRndA, encRndA, rndlen, key->data, IV, 2);
-        else if (payload->algo == MFDES_ALGO_3K3DES)
+        } else if (payload->algo == MFDES_ALGO_3K3DES) {
             tdes_nxp_receive(encRndA, encRndA, rndlen, key->data, IV, 3);
+        }
+
     } else if (payload->mode == MFDES_AUTH_AES) {
+
         if (mbedtls_aes_setkey_dec(&ctx, key->data, 128) != 0) {
-            if (DBGLEVEL >= DBG_EXTENDED) {
+            if (g_dbglevel >= DBG_EXTENDED) {
                 DbpString("mbedtls_aes_setkey_dec failed");
             }
             OnErrorNG(CMD_HF_DESFIRE_AUTH1, 7);
@@ -498,11 +562,12 @@ void MifareDES_Auth1(uint8_t *datain) {
     }
 
     rol(RndA, rndlen);
-    if (DBGLEVEL >= DBG_EXTENDED) {
+    if (g_dbglevel >= DBG_EXTENDED) {
         print_result("RndA : ", RndA, rndlen);
         print_result("RndB: ", RndB, rndlen);
         print_result("encRndA : ", encRndA, rndlen);
     }
+
     for (int x = 0; x < rndlen; x++) {
         if (RndA[x] != encRndA[x]) {
             DbpString("Authentication failed. Cannot verify Session Key.");
@@ -549,7 +614,7 @@ void MifareDES_Auth1(uint8_t *datain) {
      memcpy(cmd+19,buff3,8);
 
      // The command always times out on the first attempt, this will retry until a response
-     // is recieved.
+     // is received.
      len = 0;
      while(!len) {
      len = DesfireAPDU(cmd,27,resp);
@@ -592,7 +657,7 @@ void MifareDES_Auth1(uint8_t *datain) {
     memcpy(cmd+19,buff3,8);
 
     // The command always times out on the first attempt, this will retry until a response
-    // is recieved.
+    // is received.
     len = 0;
     while(!len) {
         len = DesfireAPDU(cmd,27,resp);
@@ -602,10 +667,6 @@ void MifareDES_Auth1(uint8_t *datain) {
     */
 
 
-    //OnSuccess();
-    //reply_old(CMD_ACK, 1, 0, 0, skey->data, payload->keylen);
-    //reply_mix(CMD_ACK, 1, len, 0, resp, len);
-
     LED_B_ON();
     authres_t rpayload;
     rpayload.sessionkeylen = payload->keylen;
@@ -628,15 +689,17 @@ int DesfireAPDU(uint8_t *cmd, size_t cmd_len, uint8_t *dataout) {
 
     wrappedLen = CreateAPDU(cmd, cmd_len, wCmd);
 
-    if (DBGLEVEL >= DBG_EXTENDED)
+    if (g_dbglevel >= DBG_EXTENDED) {
         print_result("WCMD <--: ", wCmd, wrappedLen);
+    }
 
     ReaderTransmit(wCmd, wrappedLen, NULL);
 
-    len = ReaderReceive(resp, par);
-    if (!len) {
-        if (DBGLEVEL >= DBG_EXTENDED) Dbprintf("fukked");
+    len = ReaderReceive(resp, sizeof(resp), par);
+    if (len == 0) {
+        if (g_dbglevel >= DBG_EXTENDED) Dbprintf("fukked");
         return false; //DATA LINK ERROR
+
     }
     // if we received an I- or R(ACK)-Block with a block number equal to the
     // current block number, toggle the current block number
@@ -662,7 +725,7 @@ size_t CreateAPDU(uint8_t *datain, size_t len, uint8_t *dataout) {
     cmd[0] = 0x02;  //  0x0A = send cid,  0x02 = no cid.
     cmd[0] |= pcb_blocknum; // OR the block number into the PCB
 
-    if (DBGLEVEL >= DBG_EXTENDED) Dbprintf("pcb_blocknum %d == %d ", pcb_blocknum, cmd[0]);
+    if (g_dbglevel >= DBG_EXTENDED) Dbprintf("pcb_blocknum %d == %d ", pcb_blocknum, cmd[0]);
 
     //cmd[1] = 0x90;  //  CID: 0x00 //TODO: allow multiple selected cards
 
@@ -687,7 +750,7 @@ void OnSuccess(void) {
     pcb_blocknum = 0;
     ReaderTransmit(deselect_cmd, 3, NULL);
     if (mifare_ultra_halt()) {
-        if (DBGLEVEL >= DBG_ERROR) Dbprintf("Halt error");
+        if (g_dbglevel >= DBG_ERROR) Dbprintf("Halt error");
     }
     switch_off();
 }
