@@ -1,9 +1,20 @@
-/*
- * spiffs_hydrogen.c
- *
- *  Created on: Jun 16, 2013
- *      Author: petera
- */
+//-----------------------------------------------------------------------------
+// Borrowed initially from https://github.com/pellepl/spiffs
+// Copyright (c) 2013-2017 Peter Andersson (pelleplutt1976 at gmail.com)
+// Copyright (C) Proxmark3 contributors. See AUTHORS.md for details.
+//
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// See LICENSE.txt for the text of the license.
+//-----------------------------------------------------------------------------
 
 #include "spiffs.h"
 #include "spiffs_nucleus.h"
@@ -39,13 +50,20 @@ s32_t SPIFFS_format(spiffs *fs) {
         return -1;
     }
 
-    s32_t res;
     SPIFFS_LOCK(fs);
 
+    uint32_t block_count = fs->block_count;
+    // this _should_ never happen, but prefer to see debug message / error
+    // rather than silently entering infinite loop.
+    if (block_count > ((spiffs_block_ix)(-1))) {
+        SPIFFS_DBG("Avoiding infinite loop, block_count "_SPIPRIbl" too large for spiffs_block_ix type\n", block_count);
+        SPIFFS_API_CHECK_RES_UNLOCK(fs, SPIFFS_ERR_INTERNAL);
+    }
+
     spiffs_block_ix bix = 0;
-    while (bix < fs->block_count) {
+    while (bix < block_count) {
         fs->max_erase_count = 0;
-        res = spiffs_erase_block(fs, bix);
+        s32_t res = spiffs_erase_block(fs, bix);
         if (res != SPIFFS_OK) {
             res = SPIFFS_ERR_ERASE_FAIL;
         }
@@ -1121,11 +1139,11 @@ s32_t SPIFFS_check(spiffs *fs) {
     SPIFFS_LOCK(fs);
 
     res = spiffs_lookup_consistency_check(fs, 0);
-
+    (void)res;
     res = spiffs_object_index_consistency_check(fs);
-
+    (void)res;
     res = spiffs_page_consistency_check(fs);
-
+    (void)res;
     res = spiffs_obj_lu_scan(fs);
 
     SPIFFS_UNLOCK(fs);
@@ -1445,7 +1463,7 @@ s32_t SPIFFS_vis(spiffs *fs) {
         if (erase_count != (spiffs_obj_id) - 1) {
             spiffs_printf("\tera_cnt: "_SPIPRIi"\n", erase_count);
         } else {
-            spiffs_printf("\tera_cnt: N/A\n");
+            spiffs_printf("\tera_cnt: n/a\n");
         }
 
         bix++;
