@@ -1,17 +1,22 @@
-/*
- * libopenemv - a library to work with EMV family of smart cards
- * Copyright (C) 2015 Dmitry Eremin-Solenikov
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- */
+//-----------------------------------------------------------------------------
+// Borrowed initially from https://github.com/lumag/emv-tools/
+// Copyright (C) 2012, 2015 Dmitry Eremin-Solenikov
+// Copyright (C) Proxmark3 contributors. See AUTHORS.md for details.
+//
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// See LICENSE.txt for the text of the license.
+//-----------------------------------------------------------------------------
+// libopenemv - a library to work with EMV family of smart cards
+//-----------------------------------------------------------------------------
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -24,15 +29,15 @@
 #include <stdarg.h>
 
 #include "crypto.h"
-#include "dump.h"
 #include "util.h"
+#include "ui.h"
 
 static bool strictExecution = true;
 void PKISetStrictExecution(bool se) {
     strictExecution = se;
 }
 
-static const unsigned char empty_tlv_value[] = {};
+static const unsigned char empty_tlv_value[] = {0};
 static const struct tlv empty_tlv = {.tag = 0x0, .len = 0, .value = empty_tlv_value};
 
 static size_t emv_pki_hash_psn[256] = { 0, 0, 11, 2, 17, 2, };
@@ -53,12 +58,12 @@ static unsigned char *emv_pki_decode_message(const struct emv_pk *enc_pk,
         return NULL;
 
     if (!cert_tlv) {
-        printf("ERROR: Can't find certificate\n");
+        PrintAndLogEx(WARNING, "ERROR: Can't find certificate");
         return NULL;
     }
 
     if (cert_tlv->len != enc_pk->mlen) {
-        printf("ERROR: Certificate length (%zu) not equal key length (%zu)\n", cert_tlv->len, enc_pk->mlen);
+        PrintAndLogEx(WARNING, "ERROR: Certificate length (%zu) not equal key length (%zu)", cert_tlv->len, enc_pk->mlen);
         return NULL;
     }
     kcp = crypto_pk_open(enc_pk->pk_algo,
@@ -71,19 +76,19 @@ static unsigned char *emv_pki_decode_message(const struct emv_pk *enc_pk,
     crypto_pk_close(kcp);
 
     /*  if (true){
-            printf("Recovered data:\n");
-            dump_buffer(data, data_len, stdout, 0);
+            PrintAndLogEx(SUCCESS, "Recovered data:\n");
+            print_buffer(data, data_len, 1);
         }*/
 
     if (data[data_len - 1] != 0xbc || data[0] != 0x6a || data[1] != msgtype) {
-        printf("ERROR: Certificate format\n");
+        PrintAndLogEx(WARNING, "ERROR: Certificate format");
         free(data);
         return NULL;
     }
 
     size_t hash_pos = emv_pki_hash_psn[msgtype];
     if (hash_pos == 0 || hash_pos > data_len) {
-        printf("ERROR: Cant get hash position in the certificate\n");
+        PrintAndLogEx(WARNING, "ERROR: Cant get hash position in the certificate");
         free(data);
         return NULL;
     }
@@ -91,7 +96,7 @@ static unsigned char *emv_pki_decode_message(const struct emv_pk *enc_pk,
     struct crypto_hash *ch;
     ch = crypto_hash_open(data[hash_pos]);
     if (!ch) {
-        printf("ERROR: Cant do hash\n");
+        PrintAndLogEx(WARNING, "ERROR: Cant do hash");
         free(data);
         return NULL;
     }
@@ -113,9 +118,9 @@ static unsigned char *emv_pki_decode_message(const struct emv_pk *enc_pk,
     memset(hash, 0, hash_len);
     memcpy(hash, crypto_hash_read(ch), hash_len);
     if (memcmp(data + data_len - 1 - hash_len, hash, hash_len)) {
-        printf("ERROR: Calculated wrong hash\n");
-        printf("decoded:    %s\n", sprint_hex(data + data_len - 1 - hash_len, hash_len));
-        printf("calculated: %s\n", sprint_hex(hash, hash_len));
+        PrintAndLogEx(WARNING, "ERROR: Calculated wrong hash");
+        PrintAndLogEx(WARNING, "decoded:    " _YELLOW_("%s"), sprint_hex(data + data_len - 1 - hash_len, hash_len));
+        PrintAndLogEx(WARNING, "calculated: " _YELLOW_("%s"), sprint_hex(hash, hash_len));
 
         if (strictExecution) {
             crypto_hash_close(ch);
@@ -125,16 +130,12 @@ static unsigned char *emv_pki_decode_message(const struct emv_pk *enc_pk,
     }
 
     crypto_hash_close(ch);
-
     *len = data_len - hash_len - 1;
-
     return data;
 }
 
 static unsigned emv_cn_length(const struct tlv *tlv) {
-    int i;
-
-    for (i = 0; i < tlv->len; i++) {
+    for (int i = 0; i < tlv->len; i++) {
         unsigned char c = tlv->value[i];
 
         if (c >> 4 == 0xf)
@@ -143,7 +144,6 @@ static unsigned emv_cn_length(const struct tlv *tlv) {
         if ((c & 0xf) == 0xf)
             return 2 * i + 1;
     }
-
     return 2 * tlv->len;
 }
 
@@ -185,7 +185,7 @@ static struct emv_pk *emv_pki_decode_key_ex(const struct emv_pk *enc_pk,
     else if (msgtype == 4)
         pan_length = 10;
     else {
-        printf("ERROR: Message type must be 2 or 4\n");
+        PrintAndLogEx(WARNING, "ERROR: Message type must be 2 or 4");
         return NULL;
     }
 
@@ -196,15 +196,15 @@ static struct emv_pk *emv_pki_decode_key_ex(const struct emv_pk *enc_pk,
                                   exp_tlv,
                                   add_tlv,
                                   sdatl_tlv,
-                                  NULL);
+                                  (uint8_t *)NULL);
     if (!data || data_len < 11 + pan_length) {
-        printf("ERROR: Can't decode message\n");
+        PrintAndLogEx(WARNING, "ERROR: Can't decode message");
         return NULL;
     }
 
     if (showData) {
-        printf("Recovered data:\n");
-        dump_buffer(data, data_len, stdout, 0);
+        PrintAndLogEx(SUCCESS, "Recovered data:");
+        print_buffer(data, data_len, 1);
     }
 
     /* Perform the rest of checks here */
@@ -219,7 +219,7 @@ static struct emv_pk *emv_pki_decode_key_ex(const struct emv_pk *enc_pk,
 
     if (((msgtype == 2) && (pan2_len < 4 || pan2_len > pan_len)) ||
             ((msgtype == 4) && (pan2_len != pan_len))) {
-        printf("ERROR: Invalid PAN lengths\n");
+        PrintAndLogEx(WARNING, "ERROR: Invalid PAN lengths");
         free(data);
 
         return NULL;
@@ -228,9 +228,9 @@ static struct emv_pk *emv_pki_decode_key_ex(const struct emv_pk *enc_pk,
     unsigned i;
     for (i = 0; i < pan2_len; i++)
         if (emv_cn_get(pan_tlv, i) != emv_cn_get(&pan2_tlv, i)) {
-            printf("ERROR: PAN data mismatch\n");
-            printf("tlv  pan=%s\n", sprint_hex(pan_tlv->value, pan_tlv->len));
-            printf("cert pan=%s\n", sprint_hex(pan2_tlv.value, pan2_tlv.len));
+            PrintAndLogEx(WARNING, "ERROR: PAN data mismatch");
+            PrintAndLogEx(WARNING, "tlv  pan " _YELLOW_("%s"), sprint_hex(pan_tlv->value, pan_tlv->len));
+            PrintAndLogEx(WARNING, "cert pan " _YELLOW_("%s"), sprint_hex(pan2_tlv.value, pan2_tlv.len));
             free(data);
 
             return NULL;
@@ -238,7 +238,7 @@ static struct emv_pk *emv_pki_decode_key_ex(const struct emv_pk *enc_pk,
 
     pk_len = data[9 + pan_length];
     if (pk_len > data_len - 11 - pan_length + rem_tlv->len) {
-        printf("ERROR: Invalid pk length\n");
+        PrintAndLogEx(WARNING, "ERROR: Invalid pk length");
         free(data);
         return NULL;
     }
@@ -268,7 +268,6 @@ static struct emv_pk *emv_pki_decode_key_ex(const struct emv_pk *enc_pk,
     memcpy(pk->exp, exp_tlv->value, exp_tlv->len);
 
     free(data);
-
     return pk;
 }
 
@@ -311,7 +310,7 @@ struct emv_pk *emv_pki_recover_icc_cert(const struct emv_pk *pk, struct tlvdb *d
                                             sda_tlv,
                                             &sda_tdata);
 
-    free(sdatl); // malloc here: emv_pki_sdatl_fill
+    free(sdatl); // calloc here: emv_pki_sdatl_fill
     return res;
 }
 
@@ -346,7 +345,11 @@ unsigned char *emv_pki_sdatl_fill(const struct tlvdb *db, size_t *sdatl_len) {
 
     if (len) {
         *sdatl_len = len;
-        unsigned char *value = malloc(len);
+        unsigned char *value = calloc(1, len);
+        if (value == NULL) {
+            PrintAndLogEx(WARNING, "Failed to allocate memory");
+            return NULL;
+        }
         memcpy(value, buf, len);
         return value;
     }
@@ -354,9 +357,8 @@ unsigned char *emv_pki_sdatl_fill(const struct tlvdb *db, size_t *sdatl_len) {
     return NULL;
 }
 
-
 struct tlvdb *emv_pki_recover_dac_ex(const struct emv_pk *enc_pk, const struct tlvdb *db, const struct tlv *sda_tlv, bool showData) {
-    size_t data_len;
+    size_t data_len = 0;
 
     // Static Data Authentication Tag List
     size_t sdatl_len;
@@ -372,24 +374,23 @@ struct tlvdb *emv_pki_recover_dac_ex(const struct emv_pk *enc_pk, const struct t
                                                  3,
                                                  sda_tlv,
                                                  &sda_tdata,
-                                                 NULL);
+                                                 (uint8_t *)NULL);
 
-    free(sdatl); // malloc here: emv_pki_sdatl_fill
+    free(sdatl); // calloc here: emv_pki_sdatl_fill
 
     if (!data || data_len < 5)
         return NULL;
 
     if (showData) {
-        printf("Recovered data:\n");
-        dump_buffer(data, data_len, stdout, 0);
+        PrintAndLogEx(SUCCESS, "Recovered data:");
+        print_buffer(data, data_len, 1);
     }
 
     struct tlvdb *dac_db = tlvdb_fixed(0x9f45, 2, data + 3);
-
     free(data);
-
     return dac_db;
 }
+
 struct tlvdb *emv_pki_recover_dac(const struct emv_pk *enc_pk, const struct tlvdb *db, const struct tlv *sda_tlv) {
     return emv_pki_recover_dac_ex(enc_pk, db, sda_tlv, false);
 }
@@ -404,7 +405,7 @@ struct tlvdb *emv_pki_recover_idn_ex(const struct emv_pk *enc_pk, const struct t
                                                  tlvdb_get(db, 0x9f4b, NULL),
                                                  2,
                                                  dyn_tlv,
-                                                 NULL);
+                                                 (uint8_t *)NULL);
 
     if (!data || data_len < 3)
         return NULL;
@@ -415,8 +416,8 @@ struct tlvdb *emv_pki_recover_idn_ex(const struct emv_pk *enc_pk, const struct t
     }
 
     if (showData) {
-        printf("Recovered data:\n");
-        dump_buffer(data, data_len, stdout, 0);
+        PrintAndLogEx(SUCCESS, "Recovered data:");
+        print_buffer(data, data_len, 1);
     }
 
     size_t idn_len = data[4];
@@ -427,9 +428,7 @@ struct tlvdb *emv_pki_recover_idn_ex(const struct emv_pk *enc_pk, const struct t
 
     // 9f4c ICC Dynamic Number
     struct tlvdb *idn_db = tlvdb_fixed(0x9f4c, idn_len, data + 5);
-
     free(data);
-
     return idn_db;
 }
 
@@ -442,7 +441,7 @@ struct tlvdb *emv_pki_recover_atc_ex(const struct emv_pk *enc_pk, const struct t
                                                  tlvdb_get(db, 0x9f02, NULL),
                                                  tlvdb_get(db, 0x5f2a, NULL),
                                                  tlvdb_get(db, 0x9f69, NULL),
-                                                 NULL);
+                                                 (uint8_t *)NULL);
 
     if (!data || data_len < 3)
         return NULL;
@@ -453,8 +452,8 @@ struct tlvdb *emv_pki_recover_atc_ex(const struct emv_pk *enc_pk, const struct t
     }
 
     if (showData) {
-        printf("Recovered data:\n");
-        dump_buffer(data, data_len, stdout, 0);
+        PrintAndLogEx(SUCCESS, "Recovered data:");
+        print_buffer(data, data_len, 1);
     }
 
     size_t idn_len = data[4];
@@ -471,22 +470,20 @@ struct tlvdb *emv_pki_recover_atc_ex(const struct emv_pk *enc_pk, const struct t
     return atc_db;
 }
 
-static bool tlv_hash(void *data, const struct tlv *tlv, int level, bool is_leaf) {
+static void tlv_hash(void *data, const struct tlv *tlv, int level, bool is_leaf) {
     struct crypto_hash *ch = data;
     size_t tag_len;
     unsigned char *tag;
 
     if (tlv_is_constructed(tlv))
-        return true;
+        return;
 
     if (tlv->tag == 0x9f4b)
-        return true;
+        return;
 
     tag = tlv_encode(tlv, &tag_len);
     crypto_hash_write(ch, tag, tag_len);
     free(tag);
-
-    return true;
 }
 
 struct tlvdb *emv_pki_perform_cda(const struct emv_pk *enc_pk, const struct tlvdb *db,
@@ -496,6 +493,7 @@ struct tlvdb *emv_pki_perform_cda(const struct emv_pk *enc_pk, const struct tlvd
                                   const struct tlv *crm2_tlv) {
     return emv_pki_perform_cda_ex(enc_pk, db, this_db, pdol_data_tlv, crm1_tlv, crm2_tlv, false);
 }
+
 struct tlvdb *emv_pki_perform_cda_ex(const struct emv_pk *enc_pk, const struct tlvdb *db,
                                      const struct tlvdb *this_db,     // AC TLV result
                                      const struct tlv *pdol_data_tlv, // PDOL
@@ -513,25 +511,25 @@ struct tlvdb *emv_pki_perform_cda_ex(const struct emv_pk *enc_pk, const struct t
                                                  tlvdb_get(this_db, 0x9f4b, NULL),
                                                  2,
                                                  un_tlv,
-                                                 NULL);
+                                                 (uint8_t *)NULL);
     if (!data || data_len < 3) {
-        printf("ERROR: can't decode message. len %zu\n", data_len);
+        PrintAndLogEx(WARNING, "ERROR: can't decode message. [%zu bytes]", data_len);
         return NULL;
     }
 
     if (showData) {
-        printf("Recovered data:\n");
-        dump_buffer(data, data_len, stdout, 0);
+        PrintAndLogEx(SUCCESS, "Recovered data:");
+        print_buffer(data, data_len, 1);
     }
 
     if (data[3] < 30 || data[3] > data_len - 4) {
-        printf("ERROR: Invalid data length\n");
+        PrintAndLogEx(WARNING, "ERROR: Invalid data length");
         free(data);
         return NULL;
     }
 
     if (!cid_tlv || cid_tlv->len != 1 || cid_tlv->value[0] != data[5 + data[4]]) {
-        printf("ERROR: CID mismatch\n");
+        PrintAndLogEx(WARNING, "ERROR: CID mismatch");
         free(data);
         return NULL;
     }
@@ -539,7 +537,7 @@ struct tlvdb *emv_pki_perform_cda_ex(const struct emv_pk *enc_pk, const struct t
     struct crypto_hash *ch;
     ch = crypto_hash_open(enc_pk->hash_algo);
     if (!ch) {
-        printf("ERROR: can't create hash\n");
+        PrintAndLogEx(WARNING, "ERROR: can't create hash");
         free(data);
         return NULL;
     }
@@ -554,7 +552,7 @@ struct tlvdb *emv_pki_perform_cda_ex(const struct emv_pk *enc_pk, const struct t
     tlvdb_visit(this_db, tlv_hash, ch, 0);
 
     if (memcmp(data + 5 + data[4] + 1 + 8, crypto_hash_read(ch), 20)) {
-        printf("ERROR: calculated hash error\n");
+        PrintAndLogEx(WARNING, "ERROR: calculated hash error");
         crypto_hash_close(ch);
         free(data);
         return NULL;
@@ -563,7 +561,7 @@ struct tlvdb *emv_pki_perform_cda_ex(const struct emv_pk *enc_pk, const struct t
 
     size_t idn_len = data[4];
     if (idn_len > data[3] - 1) {
-        printf("ERROR: Invalid IDN length\n");
+        PrintAndLogEx(WARNING, "ERROR: Invalid IDN length");
         free(data);
         return NULL;
     }
